
import json

import pytest
from unittest.mock import patch, MagicMock, AsyncMock, ANY
from aird.handlers.share_handlers import (
    ShareFilesHandler,
    ShareCreateHandler,
    ShareRevokeHandler,
    ShareUpdateHandler,
    TokenVerificationHandler,
    SharedListHandler,
    SharedFileHandler,
)
from aird.cloud import CloudProviderError

from tests.handler_helpers import authenticate, patch_db_conn, prepare_handler


class TestShareFilesHandler:
    def setup_method(self):
        self.mock_app = MagicMock()
        self.mock_request = MagicMock()
        self.mock_app.settings = {'cookie_secret': 'test_secret'}

    def test_get_share_page(self):
        handler = prepare_handler(ShareFilesHandler(self.mock_app, self.mock_request))
        authenticate(handler, role='user')

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=True), \
             patch.object(handler, 'render') as mock_render:

            handler.get()
            mock_render.assert_called_with("share.html", shares={})

    def test_get_feature_disabled(self):
        handler = prepare_handler(ShareFilesHandler(self.mock_app, self.mock_request))
        authenticate(handler, role='user')
        handler.set_status = MagicMock()

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=False), \
             patch.object(handler, 'write') as mock_write:

            handler.get()
            handler.set_status.assert_called_with(403)
            mock_write.assert_called_with("Feature disabled: File sharing is currently disabled by administrator")


class TestShareCreateHandler:
    def setup_method(self):
        self.mock_app = MagicMock()
        self.mock_request = MagicMock()
        self.mock_app.settings = {'cookie_secret': 'test_secret'}

    def _build_handler(self, body):
        handler = prepare_handler(ShareCreateHandler(self.mock_app, self.mock_request))
        authenticate(handler, role='user')
        handler.check_xsrf_cookie = MagicMock()
        handler.request.headers = {'X-XSRFToken': 'token'}
        handler.get_cookie = MagicMock(return_value='token')
        handler.request.body = json.dumps(body).encode('utf-8')
        handler.set_status = MagicMock()
        return handler

    def test_create_share_success(self):
        handler = self._build_handler({'paths': ['test.txt']})

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=True), \
             patch('os.path.abspath', return_value='/root/test.txt'), \
             patch('aird.handlers.share_handlers.is_within_root', return_value=True), \
             patch('os.path.isfile', return_value=True), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.insert_share', return_value=True), \
             patch.object(handler, 'write') as mock_write:

            handler.post()
            mock_write.assert_called()
            assert 'id' in mock_write.call_args[0][0]

    def test_create_share_feature_disabled(self):
        handler = self._build_handler({'paths': ['test.txt']})

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=False), \
             patch.object(handler, 'write') as mock_write:
            handler.post()
            handler.set_status.assert_called_with(403)
            mock_write.assert_called_with({"error": "File sharing is disabled"})

    def test_create_share_no_valid_files(self):
        handler = self._build_handler({'paths': ['missing.txt']})

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=True), \
             patch('os.path.abspath', return_value='/root/missing.txt'), \
             patch('aird.handlers.share_handlers.is_within_root', return_value=True), \
             patch('os.path.isfile', return_value=False), \
             patch('os.path.isdir', return_value=False), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.remove_share_cloud_dir') as mock_remove, \
             patch.object(handler, 'write') as mock_write:

            handler.post()
            handler.set_status.assert_called_with(400)
            mock_write.assert_called_with({"error": "No valid files or directories"})
            mock_remove.assert_called()

    def test_create_share_dynamic_requires_directory(self):
        handler = self._build_handler({'paths': [], 'share_type': 'dynamic'})

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=True), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.remove_share_cloud_dir') as mock_remove, \
             patch.object(handler, 'write') as mock_write:

            handler.post()
            handler.set_status.assert_called_with(400)
            mock_write.assert_called_with({"error": "No valid directories for dynamic share"})
            mock_remove.assert_called()

    def test_create_share_cloud_error(self):
        handler = self._build_handler({'paths': [{'type': 'cloud', 'path': 'cloud://item'}]})

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=True), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.download_cloud_items', side_effect=CloudProviderError("boom")), \
             patch('aird.handlers.share_handlers.remove_share_cloud_dir') as mock_remove, \
             patch.object(handler, 'write') as mock_write:

            handler.post()
            handler.set_status.assert_called_with(400)
            mock_write.assert_called_with({"error": "boom"})
            mock_remove.assert_called()

    def test_create_share_db_missing(self):
        handler = self._build_handler({'paths': ['file.txt']})

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=True), \
             patch('os.path.abspath', return_value='/root/file.txt'), \
             patch('aird.handlers.share_handlers.is_within_root', return_value=True), \
             patch('os.path.isfile', return_value=True), \
             patch_db_conn(None, modules=['aird.handlers.share_handlers']), \
             patch.object(handler, 'write') as mock_write:

            handler.post()
            handler.set_status.assert_called_with(500)
            mock_write.assert_called_with({"error": "Database connection not available"})


class TestShareRevokeHandler:
    def setup_method(self):
        self.mock_app = MagicMock()
        self.mock_request = MagicMock()
        self.mock_app.settings = {'cookie_secret': 'test_secret'}

    def test_revoke_share_redirect(self):
        handler = prepare_handler(ShareRevokeHandler(self.mock_app, self.mock_request))
        authenticate(handler, role='user')
        handler.get_argument = MagicMock(return_value="share1")

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=True), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.delete_share') as mock_delete, \
             patch('aird.handlers.share_handlers.remove_share_cloud_dir'), \
             patch.object(handler, 'redirect') as mock_redirect:

            handler.post()
            assert mock_delete.call_args[0][1] == "share1"
            mock_redirect.assert_called_with('/share')

    def test_revoke_share_json_response(self):
        handler = prepare_handler(ShareRevokeHandler(self.mock_app, self.mock_request))
        authenticate(handler, role='user')
        handler.get_argument = MagicMock(return_value="share1")
        handler.request.headers = {'Accept': 'application/json'}

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=True), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.delete_share'), \
             patch('aird.handlers.share_handlers.remove_share_cloud_dir'), \
             patch.object(handler, 'write') as mock_write:

            handler.post()
            mock_write.assert_called_with({'ok': True})


class TestShareUpdateHandler:
    def setup_method(self):
        self.mock_app = MagicMock()
        self.mock_request = MagicMock()
        self.mock_app.settings = {'cookie_secret': 'test_secret'}

    def _build_handler(self, body):
        handler = prepare_handler(ShareUpdateHandler(self.mock_app, self.mock_request))
        authenticate(handler, role='user')
        handler.check_xsrf_cookie = MagicMock()
        handler.request.body = json.dumps(body).encode('utf-8')
        handler.set_status = MagicMock()
        return handler

    def test_update_share_missing_id(self):
        handler = self._build_handler({})

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=True), \
             patch.object(handler, 'write') as mock_write:

            handler.post()
            handler.set_status.assert_called_with(400)
            mock_write.assert_called_with({"error": "Share ID is required"})

    def test_update_share_not_found(self):
        handler = self._build_handler({'share_id': 'missing'})

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=True), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.get_share_by_id', return_value=None), \
             patch.object(handler, 'write') as mock_write:

            handler.post()
            handler.set_status.assert_called_with(404)
            mock_write.assert_called_with({"error": "Share not found"})

    def test_update_share_db_missing(self):
        handler = self._build_handler({'share_id': 'share1'})

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=True), \
             patch_db_conn(None, modules=['aird.handlers.share_handlers']), \
             patch.object(handler, 'write') as mock_write:

            handler.post()
            handler.set_status.assert_called_with(500)
            mock_write.assert_called_with({"error": "Database connection not available"})

    def test_update_share_dynamic_cloud_error(self):
        body = {
            'share_id': 'share1',
            'paths': [{'type': 'cloud', 'path': 'cloud://file'}],
            'share_type': 'static',
        }
        handler = self._build_handler(body)

        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=True), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.get_share_by_id', return_value={'paths': [], 'share_type': 'static'}), \
             patch('aird.handlers.share_handlers.download_cloud_items', side_effect=CloudProviderError("boom")), \
             patch.object(handler, 'write') as mock_write:

            handler.post()
            handler.set_status.assert_called_with(400)
            mock_write.assert_called_with({"error": "boom"})

    def test_update_share_disable_token_generates_new(self):
        body = {
            'share_id': 'share1',
            'disable_token': False,
            'paths': [],
        }
        handler = self._build_handler(body)

        share_data = {'paths': [], 'share_type': 'static', 'secret_token': None}
        with patch('aird.handlers.share_handlers.is_feature_enabled', return_value=True), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.get_share_by_id', side_effect=[share_data, {'secret_token': 'newtoken'}]), \
             patch('aird.handlers.share_handlers.update_share', return_value=True), \
             patch.object(handler, 'write') as mock_write:

            handler.post()
            response = mock_write.call_args[0][0]
            assert response['success'] is True
            assert response['new_token'] == 'newtoken'


class TestTokenVerificationHandler:
    def setup_method(self):
        self.mock_app = MagicMock()
        self.mock_request = MagicMock()
        self.mock_app.settings = {'cookie_secret': 'test_secret'}

    def test_verify_token_success(self):
        handler = prepare_handler(TokenVerificationHandler(self.mock_app, self.mock_request))
        handler.request.body = json.dumps({'token': 'secret'}).encode('utf-8')

        with patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.get_share_by_id', return_value={'secret_token': 'secret'}), \
             patch.object(handler, 'write') as mock_write:

            handler.post("share1")
            mock_write.assert_called_with({"success": True})

    def test_verify_token_missing_token(self):
        handler = prepare_handler(TokenVerificationHandler(self.mock_app, self.mock_request))
        handler.request.body = json.dumps({}).encode('utf-8')
        handler.set_status = MagicMock()

        with patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.get_share_by_id', return_value={'secret_token': 'secret'}), \
             patch.object(handler, 'write') as mock_write:

            handler.post("share1")
            handler.set_status.assert_called_with(400)
            mock_write.assert_called_with({"error": "Token is required"})

    def test_verify_token_invalid(self):
        handler = prepare_handler(TokenVerificationHandler(self.mock_app, self.mock_request))
        handler.request.body = json.dumps({'token': 'bad'}).encode('utf-8')
        handler.set_status = MagicMock()

        with patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.get_share_by_id', return_value={'secret_token': 'secret'}), \
             patch.object(handler, 'write') as mock_write:

            handler.post("share1")
            handler.set_status.assert_called_with(403)
            mock_write.assert_called_with({"error": "Invalid token"})


class TestSharedListHandler:
    def setup_method(self):
        self.mock_app = MagicMock()
        self.mock_request = MagicMock()
        self.mock_app.settings = {'cookie_secret': 'test_secret'}

    def test_get_shared_list_static(self):
        handler = prepare_handler(SharedListHandler(self.mock_app, self.mock_request))

        share_data = {'paths': ['test.txt'], 'share_type': 'static', 'secret_token': None}
        with patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.get_share_by_id', return_value=share_data), \
             patch('aird.handlers.share_handlers.is_share_expired', return_value=False), \
             patch('aird.handlers.share_handlers.filter_files_by_patterns', return_value=['test.txt']), \
             patch.object(handler, 'render') as mock_render:

            handler.get("share1")
            mock_render.assert_called()
            assert mock_render.call_args[1]['share_id'] == "share1"

    def test_shared_list_expired(self):
        handler = prepare_handler(SharedListHandler(self.mock_app, self.mock_request))
        handler.set_status = MagicMock()

        share_data = {'paths': ['test.txt'], 'share_type': 'static'}
        with patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.get_share_by_id', return_value=share_data), \
             patch('aird.handlers.share_handlers.is_share_expired', return_value=True), \
             patch.object(handler, 'write') as mock_write:

            handler.get("share1")
            handler.set_status.assert_called_with(410)
            mock_write.assert_called_with("Share expired: This share is no longer available")

    def test_shared_list_requires_token_redirects(self):
        handler = prepare_handler(SharedListHandler(self.mock_app, self.mock_request))
        handler.redirect = MagicMock()
        # Ensure get_cookie returns None (not a MagicMock) to simulate missing cookie
        handler.get_cookie = MagicMock(return_value=None)
        # Mock request.headers.get to return empty string for Authorization header
        handler.request.headers = MagicMock()
        handler.request.headers.get = MagicMock(return_value='')

        share_data = {'paths': ['test.txt'], 'share_type': 'static', 'secret_token': 'abc'}
        with patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.get_share_by_id', return_value=share_data), \
             patch('aird.handlers.share_handlers.is_share_expired', return_value=False):

            handler.get("share1")
            handler.redirect.assert_called_with("/shared/share1/verify")


class TestSharedFileHandler:
    def setup_method(self):
        self.mock_app = MagicMock()
        self.mock_request = MagicMock()
        self.mock_app.settings = {'cookie_secret': 'test_secret'}

    @pytest.mark.asyncio
    async def test_get_shared_file_success(self):
        handler = prepare_handler(SharedFileHandler(self.mock_app, self.mock_request))
        handler.set_status = MagicMock()

        share_data = {'paths': ['test.txt'], 'share_type': 'static', 'secret_token': None}
        with patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.get_share_by_id', return_value=share_data), \
             patch('aird.handlers.share_handlers.is_share_expired', return_value=False), \
             patch('aird.handlers.share_handlers.filter_files_by_patterns', return_value=['test.txt']), \
             patch('os.path.abspath', return_value='/root/test.txt'), \
             patch('aird.handlers.share_handlers.is_within_root', return_value=True), \
             patch('os.path.isfile', return_value=True), \
             patch('aird.handlers.share_handlers.MainHandler') as mock_main_handler:

            mock_main_handler.serve_file = AsyncMock()

            await handler.get("share1", "test.txt")
            mock_main_handler.serve_file.assert_awaited_with(handler, '/root/test.txt')

    @pytest.mark.asyncio
    async def test_shared_file_requires_token(self):
        handler = prepare_handler(SharedFileHandler(self.mock_app, self.mock_request))
        handler.set_status = MagicMock()
        handler.write = MagicMock()
        # Ensure get_cookie returns None (not a MagicMock)
        handler.get_cookie = MagicMock(return_value=None)
        # Mock request.headers.get to return empty string for Authorization header
        handler.request.headers = MagicMock()
        handler.request.headers.get = MagicMock(return_value='')

        share_data = {'paths': ['test.txt'], 'share_type': 'static', 'secret_token': 'secret'}
        with patch_db_conn(MagicMock(), modules=['aird.handlers.share_handlers']), \
             patch('aird.handlers.share_handlers.get_share_by_id', return_value=share_data), \
             patch('aird.handlers.share_handlers.is_share_expired', return_value=False):

            await handler.get("share1", "test.txt")
            handler.set_status.assert_called_with(403)
            handler.write.assert_called_with("Access denied: Invalid or expired access token")




import pytest
from unittest.mock import patch, MagicMock
import hashlib
import secrets
from aird.db import hash_password, verify_password

def test_argon2_hashing():
    """Test that Argon2 is used when available"""
    # Assuming ARGON2_AVAILABLE is True in this environment
    password = "secure_password"
    hashed = hash_password(password)
    assert hashed.startswith("$argon2")
    assert verify_password(password, hashed)
    assert not verify_password("wrong_password", hashed)

def test_scrypt_fallback():
    """Test Scrypt fallback when Argon2 is not available"""
    with patch('aird.db.ARGON2_AVAILABLE', False):
        password = "secure_password"
        # Force reload or adjust the module variable if needed. 
        # Since we patched the imported name in aird.db (if it was imported that way), 
        # or we patch it where it is used.
        # However, aird.db imports it. Let's patch 'aird.db.ARGON2_AVAILABLE'.
        
        hashed = hash_password(password)
        assert hashed.startswith("scrypt:")
        assert verify_password(password, hashed)
        assert not verify_password("wrong_password", hashed)

def test_legacy_sha256_verification():
    """Test that legacy SHA-256 hashes can still be verified"""
    password = "legacy_password"
    salt = secrets.token_hex(32)
    pwd_hash = hashlib.sha256((salt + password).encode('utf-8')).hexdigest()
    legacy_hash = f"{salt}:{pwd_hash}"
    
    assert verify_password(password, legacy_hash)
    assert not verify_password("wrong_password", legacy_hash)

def test_verify_password_edge_cases():
    assert not verify_password("pass", None)
    assert not verify_password("pass", "")
    assert not verify_password("pass", "invalid:format:too:many:colons")



"""Database initialization and connection management."""

import os
import sys
import sqlite3


# Global database connection
_DB_CONN = None
_DB_PATH = None


def get_data_dir() -> str:
    """Return OS-appropriate data directory for storing the SQLite DB."""
    try:
        if os.name == 'nt':  # Windows
            base = os.environ.get('LOCALAPPDATA') or os.environ.get('APPDATA') or os.path.expanduser('~\\AppData\\Local')
        elif sys.platform == 'darwin':  # macOS
            base = os.path.expanduser('~/Library/Application Support')
        else:  # Linux and others
            base = os.environ.get('XDG_DATA_HOME') or os.path.expanduser('~/.local/share')
        data_dir = os.path.join(base, 'aird')
        os.makedirs(data_dir, exist_ok=True)
        return data_dir
    except Exception:
        # Fallback to current directory
        return os.getcwd()


def init_db(conn: sqlite3.Connection) -> None:
    """Initialize database tables and migrations."""
    conn.execute(
        """
        CREATE TABLE IF NOT EXISTS feature_flags (
            key TEXT PRIMARY KEY,
            value INTEGER NOT NULL
        )
        """
    )
    conn.execute(
        """
        CREATE TABLE IF NOT EXISTS shares (
            id TEXT PRIMARY KEY,
            created TEXT NOT NULL,
            paths TEXT NOT NULL,
            allowed_users TEXT
        )
        """
    )
    conn.execute(
        """
        CREATE TABLE IF NOT EXISTS users (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            username TEXT UNIQUE NOT NULL,
            password_hash TEXT NOT NULL,
            role TEXT NOT NULL DEFAULT 'user',
            created_at TEXT NOT NULL,
            active INTEGER NOT NULL DEFAULT 1,
            last_login TEXT
        )
        """
    )
    conn.execute(
        """
        CREATE TABLE IF NOT EXISTS ldap_configs (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            name TEXT UNIQUE NOT NULL,
            server TEXT NOT NULL,
            ldap_base_dn TEXT NOT NULL,
            ldap_member_attributes TEXT NOT NULL DEFAULT 'member',
            user_template TEXT NOT NULL,
            created_at TEXT NOT NULL,
            active INTEGER NOT NULL DEFAULT 1
        )
        """
    )
    conn.execute(
        """
        CREATE TABLE IF NOT EXISTS ldap_sync_log (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            config_id INTEGER NOT NULL,
            sync_type TEXT NOT NULL,
            users_found INTEGER NOT NULL,
            users_created INTEGER NOT NULL,
            users_removed INTEGER NOT NULL,
            sync_time TEXT NOT NULL,
            status TEXT NOT NULL,
            error_message TEXT,
            FOREIGN KEY (config_id) REFERENCES ldap_configs (id)
        )
        """
    )

    # Migration for shares table
    cursor = conn.cursor()
    cursor.execute("PRAGMA table_info(shares)")
    columns = [column[1] for column in cursor.fetchall()]
    if "allowed_users" not in columns:
        cursor.execute("ALTER TABLE shares ADD COLUMN allowed_users TEXT")
    if "secret_token" not in columns:
        cursor.execute("ALTER TABLE shares ADD COLUMN secret_token TEXT")
    if "share_type" not in columns:
        cursor.execute("ALTER TABLE shares ADD COLUMN share_type TEXT DEFAULT 'static'")
    if "allow_list" not in columns:
        cursor.execute("ALTER TABLE shares ADD COLUMN allow_list TEXT")
    if "avoid_list" not in columns:
        cursor.execute("ALTER TABLE shares ADD COLUMN avoid_list TEXT")
    if "expiry_date" not in columns:
        cursor.execute("ALTER TABLE shares ADD COLUMN expiry_date TEXT")

    conn.commit()


def set_db_conn(conn: sqlite3.Connection, db_path: str = None):
    """Set the global database connection."""
    global _DB_CONN, _DB_PATH
    _DB_CONN = conn
    _DB_PATH = db_path


def get_db_conn() -> sqlite3.Connection:
    """Get the global database connection."""
    return _DB_CONN


def get_db_path() -> str:
    """Get the database file path."""
    return _DB_PATH



"""Tests for aird/core/mmap_handler.py"""

import pytest
import os
import tempfile
from unittest.mock import patch, MagicMock

from aird.core.mmap_handler import (
    MMapFileHandler,
    get_files_in_directory,
    is_video_file,
    is_audio_file,
)


class TestMMapFileHandlerShouldUseMmap:
    """Tests for MMapFileHandler.should_use_mmap"""
    
    def test_small_file_no_mmap(self):
        """Test that small files don't use mmap"""
        # Default MMAP_MIN_SIZE is 1MB (1024 * 1024)
        assert MMapFileHandler.should_use_mmap(100) is False
        assert MMapFileHandler.should_use_mmap(1000) is False
        assert MMapFileHandler.should_use_mmap(500000) is False
    
    def test_large_file_uses_mmap(self):
        """Test that large files use mmap"""
        # Files >= MMAP_MIN_SIZE should use mmap
        from aird.constants import MMAP_MIN_SIZE
        
        assert MMapFileHandler.should_use_mmap(MMAP_MIN_SIZE) is True
        assert MMapFileHandler.should_use_mmap(MMAP_MIN_SIZE + 1) is True
        assert MMapFileHandler.should_use_mmap(MMAP_MIN_SIZE * 10) is True
    
    def test_boundary_condition(self):
        """Test boundary condition at MMAP_MIN_SIZE"""
        from aird.constants import MMAP_MIN_SIZE
        
        assert MMapFileHandler.should_use_mmap(MMAP_MIN_SIZE - 1) is False
        assert MMapFileHandler.should_use_mmap(MMAP_MIN_SIZE) is True


class TestMMapFileHandlerFindLineOffsets:
    """Tests for MMapFileHandler.find_line_offsets"""
    
    def test_find_offsets_simple_file(self):
        """Test finding line offsets in a simple file"""
        with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.txt') as f:
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3\n")
            temp_path = f.name
        
        try:
            offsets = MMapFileHandler.find_line_offsets(temp_path)
            
            # Should have offsets for 3 lines
            assert len(offsets) >= 3
            assert 0 in offsets  # First line starts at 0
        finally:
            os.unlink(temp_path)
    
    def test_find_offsets_empty_file(self):
        """Test finding line offsets in an empty file"""
        with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.txt') as f:
            temp_path = f.name
        
        try:
            offsets = MMapFileHandler.find_line_offsets(temp_path)
            # Empty file should have at least offset 0
            assert 0 in offsets or len(offsets) == 0
        finally:
            os.unlink(temp_path)
    
    def test_find_offsets_with_max_lines(self):
        """Test finding line offsets with max_lines limit"""
        with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.txt') as f:
            for i in range(100):
                f.write(f"line{i}\n")
            temp_path = f.name
        
        try:
            offsets = MMapFileHandler.find_line_offsets(temp_path, max_lines=10)
            assert len(offsets) <= 10
        finally:
            os.unlink(temp_path)
    
    def test_find_offsets_single_line_no_newline(self):
        """Test file with single line and no trailing newline"""
        with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.txt') as f:
            f.write("single line without newline")
            temp_path = f.name
        
        try:
            offsets = MMapFileHandler.find_line_offsets(temp_path)
            assert 0 in offsets
        finally:
            os.unlink(temp_path)
    
    def test_find_offsets_preserves_order(self):
        """Test that offsets are in ascending order"""
        with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.txt') as f:
            f.write("line1\n")
            f.write("line2\n")
            f.write("line3\n")
            temp_path = f.name
        
        try:
            offsets = MMapFileHandler.find_line_offsets(temp_path)
            
            # Offsets should be in ascending order
            for i in range(len(offsets) - 1):
                assert offsets[i] < offsets[i + 1]
        finally:
            os.unlink(temp_path)


class TestMMapFileHandlerSearchInFile:
    """Tests for MMapFileHandler.search_in_file"""
    
    def test_search_basic_match(self):
        """Test basic search functionality"""
        with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.txt') as f:
            f.write("hello world\n")
            f.write("goodbye world\n")
            f.write("hello again\n")
            temp_path = f.name
        
        try:
            results = MMapFileHandler.search_in_file(temp_path, "hello")
            
            assert len(results) == 2
            assert results[0]['line_number'] == 1
            assert results[1]['line_number'] == 3
        finally:
            os.unlink(temp_path)
    
    def test_search_no_matches(self):
        """Test search with no matches"""
        with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.txt') as f:
            f.write("hello world\n")
            f.write("goodbye world\n")
            temp_path = f.name
        
        try:
            results = MMapFileHandler.search_in_file(temp_path, "notfound")
            assert len(results) == 0
        finally:
            os.unlink(temp_path)
    
    def test_search_with_max_results(self):
        """Test search with max_results limit"""
        with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.txt') as f:
            for i in range(100):
                f.write(f"test line {i}\n")
            temp_path = f.name
        
        try:
            results = MMapFileHandler.search_in_file(temp_path, "test", max_results=5)
            assert len(results) == 5
        finally:
            os.unlink(temp_path)
    
    def test_search_match_positions(self):
        """Test that match positions are correctly reported"""
        with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.txt') as f:
            f.write("hello hello hello\n")
            temp_path = f.name
        
        try:
            results = MMapFileHandler.search_in_file(temp_path, "hello")
            
            assert len(results) == 1
            assert len(results[0]['match_positions']) == 3
            assert 0 in results[0]['match_positions']  # First match at position 0
        finally:
            os.unlink(temp_path)
    
    def test_search_line_content_preserved(self):
        """Test that line content is correctly preserved"""
        content = "The quick brown fox"
        with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.txt') as f:
            f.write(content + "\n")
            temp_path = f.name
        
        try:
            results = MMapFileHandler.search_in_file(temp_path, "quick")
            
            assert len(results) == 1
            assert results[0]['line_content'] == content
        finally:
            os.unlink(temp_path)
    
    def test_search_empty_file(self):
        """Test search in empty file"""
        with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.txt') as f:
            temp_path = f.name
        
        try:
            results = MMapFileHandler.search_in_file(temp_path, "test")
            assert len(results) == 0
        finally:
            os.unlink(temp_path)
    
    def test_search_case_sensitive(self):
        """Test that search is case sensitive"""
        with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.txt') as f:
            f.write("Hello World\n")
            f.write("hello world\n")
            f.write("HELLO WORLD\n")
            temp_path = f.name
        
        try:
            results = MMapFileHandler.search_in_file(temp_path, "Hello")
            assert len(results) == 1
            assert results[0]['line_number'] == 1
        finally:
            os.unlink(temp_path)
    
    def test_search_unicode_content(self):
        """Test search with unicode content"""
        with tempfile.NamedTemporaryFile(mode='w', delete=False, suffix='.txt', encoding='utf-8') as f:
            f.write("Héllo wörld\n")
            f.write("日本語テスト\n")
            f.write("Normal text\n")
            temp_path = f.name
        
        try:
            results = MMapFileHandler.search_in_file(temp_path, "Héllo")
            assert len(results) == 1
            
            results = MMapFileHandler.search_in_file(temp_path, "日本語")
            assert len(results) == 1
        finally:
            os.unlink(temp_path)


class TestGetFilesInDirectory:
    """Tests for get_files_in_directory function"""
    
    def test_get_files_basic(self):
        """Test getting files in a directory"""
        with tempfile.TemporaryDirectory() as temp_dir:
            # Create test files
            with open(os.path.join(temp_dir, "file1.txt"), 'w') as f:
                f.write("content1")
            with open(os.path.join(temp_dir, "file2.txt"), 'w') as f:
                f.write("content2")
            
            files = get_files_in_directory(temp_dir)
            
            assert len(files) == 2
            names = [f['name'] for f in files]
            assert "file1.txt" in names
            assert "file2.txt" in names
    
    def test_get_files_includes_directories(self):
        """Test that directories are included in results"""
        with tempfile.TemporaryDirectory() as temp_dir:
            os.makedirs(os.path.join(temp_dir, "subdir"))
            with open(os.path.join(temp_dir, "file.txt"), 'w') as f:
                f.write("content")
            
            files = get_files_in_directory(temp_dir)
            
            dirs = [f for f in files if f['is_dir']]
            assert len(dirs) == 1
            assert dirs[0]['name'] == "subdir"
    
    def test_get_files_metadata(self):
        """Test that file metadata is included"""
        with tempfile.TemporaryDirectory() as temp_dir:
            with open(os.path.join(temp_dir, "test.txt"), 'w') as f:
                f.write("test content")
            
            files = get_files_in_directory(temp_dir)
            
            assert len(files) == 1
            file_info = files[0]
            
            assert 'name' in file_info
            assert 'is_dir' in file_info
            assert 'size_bytes' in file_info
            assert 'size_str' in file_info
            assert 'modified' in file_info
            assert 'modified_timestamp' in file_info
    
    def test_get_files_size_formatting(self):
        """Test that file size is correctly formatted"""
        with tempfile.TemporaryDirectory() as temp_dir:
            with open(os.path.join(temp_dir, "test.txt"), 'w') as f:
                f.write("x" * 2048)  # 2KB
            
            files = get_files_in_directory(temp_dir)
            
            file_info = files[0]
            assert file_info['size_bytes'] == 2048
            assert "KB" in file_info['size_str']
    
    def test_get_files_directory_size(self):
        """Test that directory size shows as dash"""
        with tempfile.TemporaryDirectory() as temp_dir:
            os.makedirs(os.path.join(temp_dir, "subdir"))
            
            files = get_files_in_directory(temp_dir)
            
            dir_info = [f for f in files if f['is_dir']][0]
            assert dir_info['size_str'] == "-"
    
    def test_get_files_empty_directory(self):
        """Test getting files in empty directory"""
        with tempfile.TemporaryDirectory() as temp_dir:
            files = get_files_in_directory(temp_dir)
            assert len(files) == 0


class TestIsVideoFile:
    """Tests for is_video_file function"""
    
    def test_common_video_extensions(self):
        """Test common video file extensions"""
        video_files = [
            "movie.mp4",
            "video.avi",
            "film.mkv",
            "clip.mov",
            "recording.wmv",
            "stream.flv",
            "web_video.webm",
            "mobile.m4v",
            "phone.3gp",
            "open.ogv"
        ]
        
        for filename in video_files:
            assert is_video_file(filename) is True, f"{filename} should be detected as video"
    
    def test_non_video_extensions(self):
        """Test non-video file extensions"""
        non_video_files = [
            "document.txt",
            "image.jpg",
            "audio.mp3",
            "archive.zip",
            "script.py",
            "data.json"
        ]
        
        for filename in non_video_files:
            assert is_video_file(filename) is False, f"{filename} should not be detected as video"
    
    def test_case_insensitive(self):
        """Test that extension detection is case insensitive"""
        assert is_video_file("movie.MP4") is True
        assert is_video_file("video.AVI") is True
        assert is_video_file("film.MkV") is True
    
    def test_no_extension(self):
        """Test file without extension"""
        assert is_video_file("noextension") is False
    
    def test_hidden_file(self):
        """Test hidden file with video extension"""
        assert is_video_file(".hidden.mp4") is True


class TestIsAudioFile:
    """Tests for is_audio_file function"""
    
    def test_common_audio_extensions(self):
        """Test common audio file extensions"""
        audio_files = [
            "song.mp3",
            "audio.wav",
            "music.flac",
            "track.aac",
            "podcast.ogg",
            "itunes.m4a",
            "windows.wma"
        ]
        
        for filename in audio_files:
            assert is_audio_file(filename) is True, f"{filename} should be detected as audio"
    
    def test_non_audio_extensions(self):
        """Test non-audio file extensions"""
        non_audio_files = [
            "document.txt",
            "video.mp4",
            "image.png",
            "archive.tar",
            "script.js"
        ]
        
        for filename in non_audio_files:
            assert is_audio_file(filename) is False, f"{filename} should not be detected as audio"
    
    def test_case_insensitive(self):
        """Test that extension detection is case insensitive"""
        assert is_audio_file("song.MP3") is True
        assert is_audio_file("audio.WAV") is True
        assert is_audio_file("music.FlAc") is True
    
    def test_no_extension(self):
        """Test file without extension"""
        assert is_audio_file("noextension") is False


class TestMMapFileHandlerServeFileChunk:
    """Tests for MMapFileHandler.serve_file_chunk (async generator)"""
    
    @pytest.mark.asyncio
    async def test_serve_small_file(self):
        """Test serving a small file (uses traditional method)"""
        content = b"Hello, World!" * 100
        
        with tempfile.NamedTemporaryFile(mode='wb', delete=False, suffix='.bin') as f:
            f.write(content)
            temp_path = f.name
        
        try:
            chunks = []
            async for chunk in MMapFileHandler.serve_file_chunk(temp_path):
                chunks.append(chunk)
            
            result = b''.join(chunks)
            assert result == content
        finally:
            os.unlink(temp_path)
    
    @pytest.mark.asyncio
    async def test_serve_with_range(self):
        """Test serving file with start/end range"""
        content = b"0123456789" * 100
        
        with tempfile.NamedTemporaryFile(mode='wb', delete=False, suffix='.bin') as f:
            f.write(content)
            temp_path = f.name
        
        try:
            chunks = []
            async for chunk in MMapFileHandler.serve_file_chunk(temp_path, start=10, end=19):
                chunks.append(chunk)
            
            result = b''.join(chunks)
            assert len(result) == 10
        finally:
            os.unlink(temp_path)
    
    @pytest.mark.asyncio
    async def test_serve_empty_file(self):
        """Test serving an empty file"""
        with tempfile.NamedTemporaryFile(mode='wb', delete=False, suffix='.bin') as f:
            temp_path = f.name
        
        try:
            chunks = []
            async for chunk in MMapFileHandler.serve_file_chunk(temp_path):
                chunks.append(chunk)
            
            result = b''.join(chunks)
            assert result == b''
        finally:
            os.unlink(temp_path)



import pytest
from unittest.mock import patch, MagicMock, AsyncMock
from aird.handlers.api_handlers import SuperSearchWebSocketHandler
import json
import asyncio

class TestSuperSearchWebSocketHandler:
    def setup_method(self):
        self.mock_app = MagicMock()
        self.mock_request = MagicMock()
        self.mock_request.path = "/search"
        self.mock_app.settings = {'cookie_secret': 'test_secret'}
        # Mock connection manager to avoid side effects
        self.mock_cm = MagicMock()
        self.mock_cm.add_connection.return_value = True
        SuperSearchWebSocketHandler.connection_manager = self.mock_cm

    @pytest.mark.asyncio
    async def test_auth_success_cookie(self):
        handler = SuperSearchWebSocketHandler(self.mock_app, self.mock_request)
        
        # Mock get_secure_cookie
        handler.get_secure_cookie = MagicMock(return_value=json.dumps({'username': 'user'}).encode())
        
        # Mock DB user check
        with patch('aird.handlers.api_handlers.constants_module.DB_CONN', MagicMock()), \
             patch('aird.db.get_user_by_username', return_value={'username': 'user'}):
            
            user = handler.get_current_user()
            assert user['username'] == 'user'

    @pytest.mark.asyncio
    async def test_auth_success_token(self):
        handler = SuperSearchWebSocketHandler(self.mock_app, self.mock_request)
        handler.get_secure_cookie = MagicMock(return_value=None)
        
        handler.request.headers = {'Authorization': 'Bearer valid_token'}
        
        with patch('aird.config.ACCESS_TOKEN', 'valid_token'):
            user = handler.get_current_user()
            assert user['username'] == 'token_user'

    @pytest.mark.asyncio
    async def test_open_auth_failed(self):
        handler = SuperSearchWebSocketHandler(self.mock_app, self.mock_request)
        handler.get_current_user = MagicMock(return_value=None)
        
        with patch.object(handler, 'close') as mock_close, \
             patch.object(handler, 'write_message') as mock_write:
            handler.open()
            mock_close.assert_called_with(code=1008, reason="Authentication required")
            mock_write.assert_called() # Sends auth_required message

    @pytest.mark.asyncio
    async def test_search_execution(self):
        handler = SuperSearchWebSocketHandler(self.mock_app, self.mock_request)
        handler.get_current_user = MagicMock(return_value={'username': 'user'})
        handler.write_message = MagicMock()
        
        # Mock os.walk and file operations
        with patch('os.walk') as mock_walk, \
             patch('pathlib.Path') as mock_path, \
             patch('builtins.open', new_callable=MagicMock) as mock_open:
            
            # Setup mock file system
            mock_walk.return_value = [('/root', [], ['file.txt'])]
            
            mock_file_path = MagicMock()
            mock_file_path.relative_to.return_value = 'file.txt'
            mock_file_path.stat.return_value.st_size = 100
            mock_path.return_value.resolve.return_value = '/root'
            mock_path.return_value.__truediv__.return_value = mock_file_path
            
            # Mock file content
            mock_file = MagicMock()
            mock_file.__enter__.return_value = ["line with search_text\n"]
            mock_open.return_value = mock_file
            
            await handler.perform_search("*.txt", "search_text")
            
            # Verify matches sent
            # write_message called for search_start, match, and done
            assert handler.write_message.call_count >= 3
            
            # Check for match message
            calls = handler.write_message.call_args_list
            match_call = next((c for c in calls if 'match' in c[0][0]), None)
            assert match_call is not None
            data = json.loads(match_call[0][0])
            assert data['type'] == 'match'
            assert data['line_content'] == 'line with search_text'

    @pytest.mark.asyncio
    async def test_on_message_start_search(self):
        handler = SuperSearchWebSocketHandler(self.mock_app, self.mock_request)
        handler.get_current_user = MagicMock(return_value={'username': 'user'})
        handler.perform_search = AsyncMock()
        
        message = json.dumps({'pattern': '*.txt', 'search_text': 'foo'})
        await handler.on_message(message)
        
        handler.perform_search.assert_called_with('*.txt', 'foo')

    @pytest.mark.asyncio
    async def test_search_no_matches(self):
        handler = SuperSearchWebSocketHandler(self.mock_app, self.mock_request)
        handler.get_current_user = MagicMock(return_value={'username': 'user'})
        handler.write_message = MagicMock()
        
        with patch('os.walk', return_value=[]), \
             patch('pathlib.Path'):
            
            await handler.perform_search("*.txt", "foo")
            
            # Check for no_matches message
            calls = handler.write_message.call_args_list
            no_match_call = next((c for c in calls if 'no_matches' in c[0][0]), None)
            assert no_match_call is not None

    @pytest.mark.asyncio
    async def test_search_cancellation(self):
        handler = SuperSearchWebSocketHandler(self.mock_app, self.mock_request)
        handler.get_current_user = MagicMock(return_value={'username': 'user'})
        handler.write_message = MagicMock()
        handler.stop_event.set()
        
        await handler.perform_search("*.txt", "foo")
        
        # Should send cancelled message
        calls = handler.write_message.call_args_list
        cancelled_call = next((c for c in calls if 'cancelled' in c[0][0]), None)
        assert cancelled_call is not None



#!/usr/bin/env python3
"""Script to combine main.py parts with handler imports"""

# Read part 1 (before handlers)
with open('aird/main_part1.txt', 'r', encoding='utf-8') as f:
    part1 = f.read()

# Read part 2 (after handlers - make_app, main, etc.)
with open('aird/main_part2.txt', 'r', encoding='utf-8') as f:
    part2 = f.read()

# Handler imports to add
handler_imports = """
# Import handlers from modules
from aird.handlers.base_handler import BaseHandler
from aird.handlers.auth_handlers import (
    LDAPLoginHandler,
    LoginHandler,
    AdminLoginHandler,
    LogoutHandler,
    ProfileHandler,
)
from aird.handlers.admin_handlers import (
    AdminHandler,
    WebSocketStatsHandler,
    AdminUsersHandler,
    UserCreateHandler,
    UserEditHandler,
    UserDeleteHandler,
    LDAPConfigHandler,
    LDAPConfigCreateHandler,
    LDAPConfigEditHandler,
    LDAPConfigDeleteHandler,
    LDAPSyncHandler,
)
from aird.handlers.file_op_handlers import (
    UploadHandler,
    DeleteHandler,
    RenameHandler,
    EditHandler,
    EditViewHandler,
    CloudUploadHandler,
)
from aird.handlers.share_handlers import (
    ShareFilesHandler,
    ShareCreateHandler,
    ShareRevokeHandler,
    ShareUpdateHandler,
    TokenVerificationHandler,
    SharedListHandler,
    SharedFileHandler,
)
from aird.handlers.api_handlers import (
    FeatureFlagSocketHandler,
    FileStreamHandler,
    FileListAPIHandler,
    ShareListAPIHandler,
    UserSearchAPIHandler,
    ShareDetailsAPIHandler,
    ShareDetailsByIdAPIHandler,
    SuperSearchHandler,
    SuperSearchWebSocketHandler,
)
from aird.handlers.view_handlers import (
    RootHandler,
    MainHandler,
    CloudProvidersHandler,
    CloudFilesHandler,
    CloudDownloadHandler,
)

"""

# Combine all parts
combined = part1 + handler_imports + part2

# Write the new main.py
with open('aird/main_new.py', 'w', encoding='utf-8') as f:
    f.write(combined)

print("Successfully created main_new.py")
print(f"Part 1: {len(part1.splitlines())} lines")
print(f"Handler imports: {len(handler_imports.splitlines())} lines")
print(f"Part 2: {len(part2.splitlines())} lines")
print(f"Total: {len(combined.splitlines())} lines")




import os
import json
import sqlite3
from datetime import datetime
import re
import weakref
import threading
import time
from urllib.parse import urlparse
import tornado.ioloop
import mmap
import chardet

MMAP_MIN_SIZE = 1024 * 1024 # 1MB
CHUNK_SIZE = 1024 * 1024 # 1MB

def _load_shares(conn: sqlite3.Connection) -> dict:
    loaded: dict = {}
    try:
        # Check if allowed_users and secret_token columns exist
        cursor = conn.execute("PRAGMA table_info(shares)")
        columns = [row[1] for row in cursor.fetchall()]
        
        if 'allowed_users' in columns and 'secret_token' in columns and 'share_type' in columns and 'allow_list' in columns and 'avoid_list' in columns and 'expiry_date' in columns:
            rows = conn.execute("SELECT id, created, paths, allowed_users, secret_token, share_type, allow_list, avoid_list, expiry_date FROM shares").fetchall()
            for sid, created, paths_json, allowed_users_json, secret_token, share_type, allow_list_json, avoid_list_json, expiry_date in rows:
                try:
                    paths = json.loads(paths_json) if paths_json else []
                except Exception:
                    paths = []
                try:
                    allowed_users = json.loads(allowed_users_json) if allowed_users_json else None
                except Exception:
                    allowed_users = None
                try:
                    allow_list = json.loads(allow_list_json) if allow_list_json else []
                except Exception:
                    allow_list = []
                try:
                    avoid_list = json.loads(avoid_list_json) if avoid_list_json else []
                except Exception:
                    avoid_list = []
                loaded[sid] = {"paths": paths, "created": created, "allowed_users": allowed_users, "secret_token": secret_token, "share_type": share_type or "static", "allow_list": allow_list, "avoid_list": avoid_list, "expiry_date": expiry_date}
        elif 'allowed_users' in columns and 'secret_token' in columns and 'share_type' in columns:
            rows = conn.execute("SELECT id, created, paths, allowed_users, secret_token, share_type FROM shares").fetchall()
            for sid, created, paths_json, allowed_users_json, secret_token, share_type in rows:
                try:
                    paths = json.loads(paths_json) if paths_json else []
                except Exception:
                    paths = []
                try:
                    allowed_users = json.loads(allowed_users_json) if allowed_users_json else None
                except Exception:
                    allowed_users = None
                loaded[sid] = {"paths": paths, "created": created, "allowed_users": allowed_users, "secret_token": secret_token, "share_type": share_type or "static", "allow_list": [], "avoid_list": [], "expiry_date": None}
        elif 'allowed_users' in columns and 'secret_token' in columns:
            rows = conn.execute("SELECT id, created, paths, allowed_users, secret_token FROM shares").fetchall()
            for sid, created, paths_json, allowed_users_json, secret_token in rows:
                try:
                    paths = json.loads(paths_json) if paths_json else []
                except Exception:
                    paths = []
                try:
                    allowed_users = json.loads(allowed_users_json) if allowed_users_json else None
                except Exception:
                    allowed_users = None
                loaded[sid] = {"paths": paths, "created": created, "allowed_users": allowed_users, "secret_token": secret_token, "share_type": "static", "allow_list": [], "avoid_list": [], "expiry_date": None}
        elif 'allowed_users' in columns:
            rows = conn.execute("SELECT id, created, paths, allowed_users FROM shares").fetchall()
            for sid, created, paths_json, allowed_users_json in rows:
                try:
                    paths = json.loads(paths_json) if paths_json else []
                except Exception:
                    paths = []
                try:
                    allowed_users = json.loads(allowed_users_json) if allowed_users_json else None
                except Exception:
                    allowed_users = None
                loaded[sid] = {"paths": paths, "created": created, "allowed_users": allowed_users, "secret_token": None, "share_type": "static", "allow_list": [], "avoid_list": [], "expiry_date": None}
        else:
            # Fallback for old schema without allowed_users column
            rows = conn.execute("SELECT id, created, paths FROM shares").fetchall()
            for sid, created, paths_json in rows:
                try:
                    paths = json.loads(paths_json) if paths_json else []
                except Exception:
                    paths = []
                loaded[sid] = {"paths": paths, "created": created, "allowed_users": None, "secret_token": None, "share_type": "static", "allow_list": [], "avoid_list": [], "expiry_date": None}
    except Exception as e:
        print(f"Error loading shares: {e}")
        import traceback
        print(f"Traceback: {traceback.format_exc()}")
        return {}
    return loaded

def get_file_icon(filename):
    ext = os.path.splitext(filename)[1].lower()
    
    # Special files by name (check first before extension)
    if filename.lower() in ["readme", "readme.md", "readme.txt"]:
        return "📖"
    elif filename.lower() in ["license", "licence", "copying"]:
        return "📜"
    elif filename.lower() in ["makefile", "cmake", "cmakelists.txt"]:
        return "🔨"
    elif filename.lower() in ["dockerfile", "docker-compose.yml", "docker-compose.yaml"]:
        return "🐳"
    elif filename.lower() in [".gitignore", ".gitattributes", ".gitmodules"]:
        return "🔧"
    elif filename.startswith(".env"):
        return "🔐"
    
    # Document files
    elif ext in [".txt", ".md", ".rst", ".text"]:
        return "📄"
    elif ext in [".doc", ".docx", ".rtf", ".odt"]:
        return "📝"
    elif ext in [".pdf"]:
        return "📕"
    elif ext in [".xls", ".xlsx", ".ods", ".csv"]:
        return "📊"
    elif ext in [".ppt", ".pptx", ".odp"]:
        return "📋"
    
    # Image files
    elif ext in [".jpg", ".jpeg", ".png", ".gif", ".bmp", ".webp", ".tiff", ".tif"]:
        return "🖼️"
    elif ext in [".svg", ".ico"]:
        return "🎨"
    elif ext in [".psd", ".ai", ".sketch"]:
        return "🎭"
    
    # Programming files
    elif ext in [".py", ".pyw"]:
        return "🐍💎"  # Enhanced Python source files with gem (precious/valuable)
    elif ext in [".pyc", ".pyo"]:
        return "🐍⚡"  # Compiled Python files with lightning (fast/optimized)
    elif ext in [".js", ".jsx", ".ts", ".tsx", ".mjs"]:
        return "🟨"
    elif ext in [".java", ".class", ".jar"]:
        return "☕"
    elif ext in [".cpp", ".cxx", ".cc", ".c", ".h", ".hpp"]:
        return "⚙️"
    elif ext in [".cs", ".vb", ".fs"]:
        return "🔷"
    elif ext in [".php", ".phtml"]:
        return "🐘"
    elif ext in [".rb", ".rake", ".gem"]:
        return "💎"
    elif ext in [".go"]:
        return "🐹"
    elif ext in [".rs"]:
        return "🦀"
    elif ext in [".swift"]:
        return "🦉"
    elif ext in [".kt", ".kts"]:
        return "🟣"
    elif ext in [".scala"]:
        return "🔴"
    elif ext in [".r", ".rmd"]:
        return "📊"
    elif ext in [".m", ".mm"]:
        return "🍎"
    elif ext in [".pl", ".pm"]:
        return "🐪"
    elif ext in [".sh", ".bash", ".zsh", ".fish", ".bat", ".cmd", ".ps1"]:
        return "📟"
    elif ext in [".lua"]:
        return "🌙"
    elif ext in [".dart"]:
        return "🎯"
    
    # Web files
    elif ext in [".html", ".htm", ".xhtml"]:
        return "🌐"
    elif ext in [".css", ".scss", ".sass", ".less"]:
        return "🎨"
    elif ext in [".xml", ".xsl", ".xsd"]:
        return "📰"
    elif ext in [".json", ".jsonl"]:
        return "📋"
    elif ext in [".yaml", ".yml"]:
        return "📄"
    elif ext in [".toml", ".ini", ".cfg", ".conf"]:
        return "⚙️"
    
    # Archive files
    elif ext in [".zip", ".rar", ".7z", ".tar", ".gz", ".bz2", ".xz", ".lz", ".lzma"]:
        return "🗜️"
    elif ext in [".deb", ".rpm", ".pkg", ".dmg", ".msi", ".exe"]:
        return "📦"
    
    # Video files
    elif ext in [".mp4", ".avi", ".mkv", ".mov", ".wmv", ".flv", ".webm", ".m4v", ".3gp", ".ogv", ".mpg", ".mpeg"]:
        return "🎬"
    
    # Audio files
    elif ext in [".mp3", ".wav", ".flac", ".aac", ".ogg", ".m4a", ".wma", ".opus", ".aiff"]:
        return "🎵"
    
    # Font files
    elif ext in [".ttf", ".otf", ".woff", ".woff2", ".eot"]:
        return "🔤"
    
    # Database files
    elif ext in [".db", ".sqlite", ".sqlite3", ".mdb", ".accdb"]:
        return "🗃️"
    
    # Log files
    elif ext in [".log", ".out", ".err"]:
        return "📜"
    
    # Data files
    elif ext in [".sql"]:
        return "🗄️"
    elif ext in [".parquet", ".avro", ".orc"]:
        return "📊"
    
    # Notebook files
    elif ext in [".ipynb"]:
        return "📓"
    
    
    # Default
    else:
        return "📦"
def join_path(*parts):
    return os.path.join(*parts).replace("\\", "/")

def is_within_root(path: str, root: str) -> bool:
    """Return True if path is within root after resolving symlinks and normalization."""
    try:
        path_real = os.path.realpath(path)
        root_real = os.path.realpath(root)
        return os.path.commonpath([path_real, root_real]) == root_real
    except Exception:
        return False

def format_size(size: int) -> str:
    """Format size in bytes to human readable string"""
    for unit in ['B', 'KB', 'MB', 'GB', 'TB']:
        if size < 1024:
            return f"{size:.2f} {unit}"
        size /= 1024
    return f"{size:.2f} PB"

def is_valid_websocket_origin(handler, origin: str) -> bool:
    """Validate WebSocket origin.
    Accept same-host origins and common localhost variants. Allow both http/https and ws/wss schemes.
    Require matching port unless the origin omits a port (treated as match).
    """
    try:
        if not origin:
            return False
        parsed = urlparse(origin)
        origin_host = parsed.hostname
        origin_port = parsed.port
        origin_scheme = (parsed.scheme or "").lower()

        if not origin_host:
            return False

        # Determine request host/port with safe defaults
        raw_host = getattr(handler.request, "host", "") or ""
        req_host = raw_host.split(":")[0] if raw_host else "localhost"
        # Default protocol http if missing on mock requests
        req_protocol = getattr(handler.request, "protocol", "http").lower()
        try:
            req_port = int(raw_host.split(":")[1])
        except (IndexError, ValueError):
            req_port = 443 if req_protocol == "https" else 80

        # Scheme check: allow http/https and ws/wss interchangeably
        allowed_schemes = {"http", "https", "ws", "wss"}
        if origin_scheme not in allowed_schemes:
            return False

        # Host check: exact match or localhost variants
        same_host = (origin_host == req_host)
        localhost_pair = origin_host in {"localhost", "127.0.0.1"} and req_host in {"localhost", "127.0.0.1"}
        if not (same_host or localhost_pair):
            return False
        return True
    except Exception:
        return False

class WebSocketConnectionManager:
    def __init__(self, config_prefix="ws", default_max_connections=100, default_idle_timeout=60):
        self.connections: set = set()
        self.config_prefix = config_prefix
        self.default_max_connections = default_max_connections
        self.default_idle_timeout = default_idle_timeout
        self.connection_times = weakref.WeakKeyDictionary()
        self.last_activity = weakref.WeakKeyDictionary()
        self._cleanup_lock = threading.Lock()
        
        # Start periodic cleanup
        self._setup_cleanup_timer()
    
    @property
    def max_connections(self) -> int:
        """Get current max connections from configuration"""
        from aird.main import get_current_websocket_config
        config = get_current_websocket_config()
        return config.get(f"{self.config_prefix}_max_connections", self.default_max_connections)
    
    @property
    def idle_timeout(self) -> int:
        """Get current idle timeout from configuration"""
        from aird.main import get_current_websocket_config
        config = get_current_websocket_config()
        return config.get(f"{self.config_prefix}_idle_timeout", self.default_idle_timeout)
    
    def _setup_cleanup_timer(self):
        """Setup periodic cleanup of dead and idle connections"""
        def cleanup():
            self.cleanup_dead_connections()
            self.cleanup_idle_connections()
            # Schedule next cleanup
            tornado.ioloop.IOLoop.current().call_later(60, cleanup)
        
        # Start cleanup in 60 seconds
        tornado.ioloop.IOLoop.current().call_later(60, cleanup)
    
    def add_connection(self, connection) -> bool:
        """Add a connection if under limit. Returns True if added."""
        with self._cleanup_lock:
            if len(self.connections) >= self.max_connections:
                return False
            
            self.connections.add(connection)
            self.connection_times[connection] = time.time()
            self.last_activity[connection] = time.time()
            return True
    
    def remove_connection(self, connection):
        """Remove a connection safely"""
        with self._cleanup_lock:
            self.connections.discard(connection)
            self.connection_times.pop(connection, None)
            self.last_activity.pop(connection, None)
    
    def update_activity(self, connection):
        """Update last activity time for a connection"""
        self.last_activity[connection] = time.time()
    
    def cleanup_dead_connections(self):
        """Remove connections that can't receive messages"""
        with self._cleanup_lock:
            dead_connections = set()
            for conn in list(self.connections):
                try:
                    # Try to ping the connection
                    if hasattr(conn, 'ws_connection') and conn.ws_connection:
                        conn.ping()
                    else:
                        # Connection is closed
                        dead_connections.add(conn)
                except Exception:
                    dead_connections.add(conn)
            
            for conn in dead_connections:
                self.remove_connection(conn)
    
    def cleanup_idle_connections(self):
        """Remove connections that have been idle too long"""
        with self._cleanup_lock:
            current_time = time.time()
            idle_connections = set()
            
            for conn in list(self.connections):
                last_activity = self.last_activity.get(conn, 0)
                if current_time - last_activity > self.idle_timeout:
                    idle_connections.add(conn)
            
            for conn in idle_connections:
                try:
                    if hasattr(conn, 'close'):
                        conn.close(code=1000, reason="Idle timeout")
                except Exception:
                    pass
                self.remove_connection(conn)
    
    def get_stats(self) -> dict:
        """Get connection statistics"""
        with self._cleanup_lock:
            current_time = time.time()
            return {
                'active_connections': len(self.connections),
                'max_connections': self.max_connections,
                'idle_timeout': self.idle_timeout,
                'oldest_connection_age': max(
                    (current_time - self.connection_times.get(conn, current_time) 
                     for conn in self.connections), 
                    default=0
                ),
                'average_connection_age': sum(
                    current_time - self.connection_times.get(conn, current_time) 
                    for conn in self.connections
                ) / len(self.connections) if self.connections else 0
            }
    
    def broadcast_message(self, message, filter_func=None):
        """Broadcast message to all connections with optional filtering"""
        with self._cleanup_lock:
            dead_connections = set()
            for conn in list(self.connections):
                try:
                    if filter_func is None or filter_func(conn):
                        if hasattr(conn, 'write_message'):
                            conn.write_message(message)
                        self.update_activity(conn)
                except Exception:
                    dead_connections.add(conn)
            
            # Remove dead connections
            for conn in dead_connections:
                self.remove_connection(conn)

class FilterExpression:
    """Parse and evaluate complex filter expressions with AND/OR logic"""
    
    def __init__(self, expression: str):
        self.original_expression = expression
        self.parsed_expression = self._parse(expression)
    
    def _parse(self, expression: str):
        """Parse filter expression into evaluable structure"""
        if not expression or not expression.strip():
            return None
            
        expression = expression.strip()
        
        # Handle escaped expressions (prefix with backslash to force literal interpretation)
        if expression.startswith('\\'):
            return {'type': 'term', 'value': expression[1:].strip('"')}
        
        # Handle quoted expressions (always literal)
        if ((expression.startswith('"') and expression.endswith('"')) or 
            (expression.startswith("'") and expression.endswith("'"))):
            return {'type': 'term', 'value': expression[1:-1]}
        
        # Check if this looks like a logical expression
        # Use word boundary regex to detect standalone AND/OR operators
        has_logical_operators = (
            re.search(r'\bAND\b', expression, re.IGNORECASE) or
            re.search(r'\bOR\b', expression, re.IGNORECASE)
        )
        
        # Additional check: make sure these are actually surrounded by whitespace (logical operators)
        if has_logical_operators:
            # Verify these are standalone words, not part of other words
            and_matches = list(re.finditer(r'\bAND\b', expression, re.IGNORECASE))
            or_matches = list(re.finditer(r'\bOR\b', expression, re.IGNORECASE))
            
            has_logical_and = any(
                self._is_standalone_operator_static(expression, match.start(), match.end())
                for match in and_matches
            )
            has_logical_or = any(
                self._is_standalone_operator_static(expression, match.start(), match.end())
                for match in or_matches
            )
            
            has_logical_operators = has_logical_and or has_logical_or
        
        if not has_logical_operators:
            return {'type': 'term', 'value': expression.strip('"')}
        
        # Parse complex expressions
        return self._parse_complex(expression)
    
    def _parse_complex(self, expression: str):
        """Parse complex expressions with AND/OR and parentheses"""
        try:
            # Handle parentheses first by balancing them
            expression = expression.strip()
            
            # If the entire expression is wrapped in parentheses, remove them
            if expression.startswith('(') and expression.endswith(')'):
                # Check if parentheses are balanced
                if self._is_balanced_parentheses(expression):
                    return self._parse_complex(expression[1:-1])
            
            # Find OR outside of parentheses (lower precedence)
            or_parts = self._split_respecting_parentheses(expression, 'OR')
            if len(or_parts) > 1:
                return {
                    'type': 'or',
                    'operands': [self._parse_and_part(part.strip()) for part in or_parts]
                }
            
            # If no OR, try AND
            return self._parse_and_part(expression)
            
        except Exception:
            # Fallback to simple term matching on parse error
            return {'type': 'term', 'value': expression.strip('"')}
    
    def _parse_and_part(self, expression: str):
        """Parse AND expressions"""
        and_parts = self._split_respecting_parentheses(expression, 'AND')
        if len(and_parts) > 1:
            return {
                'type': 'and',
                'operands': [self._parse_term(part.strip()) for part in and_parts]
            }
        return self._parse_term(expression.strip())
    
    def _parse_term(self, term: str):
        """Parse individual terms, handling quotes and parentheses"""
        term = term.strip()
        
        # Handle parentheses
        if term.startswith('(') and term.endswith(')'):
            return self._parse_complex(term[1:-1])
        
        # Handle quoted terms
        if (term.startswith('"') and term.endswith('"')) or (term.startswith("'") and term.endswith("'")):
            return {'type': 'term', 'value': term[1:-1]}
        
        return {'type': 'term', 'value': term}
    
    def matches(self, line: str) -> bool:
        """Evaluate if a line matches the filter expression"""
        if self.parsed_expression is None:
            return True
        return self._evaluate(self.parsed_expression, line)
    
    def _evaluate(self, node, line: str) -> bool:
        """Recursively evaluate parsed expression against line"""
        if node['type'] == 'term':
            return node['value'].lower() in line.lower()
        elif node['type'] == 'and':
            return all(self._evaluate(operand, line) for operand in node['operands'])
        elif node['type'] == 'or':
            return any(self._evaluate(operand, line) for operand in node['operands'])
        return False
    
    def _split_respecting_parentheses(self, expression: str, operator: str):
        """Split expression by operator while respecting parentheses and word boundaries"""
        parts = []
        current_part = ""
        paren_depth = 0
        in_quotes = False
        quote_char = None
        i = 0
        
        while i < len(expression):
            char = expression[i]
            
            # Handle quotes
            if char in ['"', "'"] and not in_quotes:
                in_quotes = True
                quote_char = char
            elif char == quote_char and in_quotes:
                in_quotes = False
                quote_char = None
            
            # Skip everything inside quotes
            if in_quotes:
                current_part += char
                i += 1
                continue
            
            # Handle parentheses
            if char == '(':
                paren_depth += 1
            elif char == ')':
                paren_depth -= 1
            
            # Check for operator when we're at the top level
            if paren_depth == 0:
                # Check if we're at the start of the operator
                remaining = expression[i:]
                # Pattern: operator with word boundaries, possibly with whitespace
                op_pattern = f'\\b{re.escape(operator)}\\b'
                match = re.match(op_pattern, remaining, re.IGNORECASE)
                if match:
                    # Verify this is actually a logical operator by checking context
                    operator_start = i
                    operator_end = i + len(match.group(0))
                    
                    # Check if surrounded by whitespace or start/end of string
                    before_ok = operator_start == 0 or expression[operator_start - 1].isspace()
                    after_ok = operator_end >= len(expression) or expression[operator_end].isspace()
                    
                    if before_ok and after_ok:
                        # Found operator at top level
                        parts.append(current_part.strip())
                        current_part = ""
                        # Skip the operator and any following whitespace
                        i = operator_end
                        while i < len(expression) and expression[i].isspace():
                            i += 1
                        continue
            
            current_part += char
            i += 1
        
        if current_part.strip():
            parts.append(current_part.strip())
        
        return parts if len(parts) > 1 else [expression]
    
    
    def _is_balanced_parentheses(self, expression: str):
        """Check if parentheses are balanced"""
        depth = 0
        in_quotes = False
        quote_char = None
        
        for char in expression:
            if char in ['"', "'"] and not in_quotes:
                in_quotes = True
                quote_char = char
            elif char == quote_char and in_quotes:
                in_quotes = False
                quote_char = None
            elif not in_quotes:
                if char == '(':
                    depth += 1
                elif char == ')':
                    depth -= 1
                    if depth < 0:
                        return False
        
        return depth == 0
    
    def _is_standalone_operator(self, expression: str, start: int, end: int, operator: str):
        """Check if AND/OR at this position is a standalone logical operator"""
        return self._is_standalone_operator_static(expression, start, end)
    
    @staticmethod
    def _is_standalone_operator_static(expression: str, start: int, end: int):
        """Static version of _is_standalone_operator for use during parsing"""
        # Check if surrounded by whitespace (indicating it's a logical operator)
        before_space = start == 0 or expression[start - 1].isspace()
        after_space = end >= len(expression) or expression[end].isspace()
        
        return before_space and after_space

    def __str__(self):
        return f"FilterExpression({self.original_expression})"

def get_files_in_directory(path="."):
    files = []
    for entry in os.scandir(path):
        stat = entry.stat()
        files.append({
            "name": entry.name,
            "is_dir": entry.is_dir(),
            "size_bytes": stat.st_size,
            "size_str": f"{stat.st_size / 1024:.2f} KB" if not entry.is_dir() else "-",
            "modified": datetime.fromtimestamp(stat.st_mtime).strftime('%Y-%m-%d %H:%M:%S'),
            "modified_timestamp": int(stat.st_mtime)
        })
    return files

def is_video_file(filename):
    """Check if file is a supported video format"""
    ext = os.path.splitext(filename)[1].lower()
    video_extensions = {'.mp4', '.avi', '.mkv', '.mov', '.wmv', '.flv', '.webm', '.m4v', '.3gp', '.ogv'}
    return ext in video_extensions

def is_audio_file(filename):
    """Check if file is a supported audio format"""
    ext = os.path.splitext(filename)[1].lower()
    audio_extensions = {'.mp3', '.wav', '.flac', '.aac', '.ogg', '.m4a', '.wma'}
    return ext in audio_extensions

def get_all_files_recursive(root_path: str, base_path: str = "") -> list:
    """Recursively get all files in a directory"""
    all_files = []
    try:
        for item in os.listdir(root_path):
            item_path = os.path.join(root_path, item)
            relative_path = os.path.join(base_path, item) if base_path else item
            
            if os.path.isfile(item_path):
                # It's a file, add it to the list
                all_files.append(relative_path)
            elif os.path.isdir(item_path):
                # It's a directory, recursively scan it
                sub_files = get_all_files_recursive(item_path, relative_path)
                all_files.extend(sub_files)
    except (OSError, PermissionError) as e:
        print(f"Error scanning directory {root_path}: {e}")
    
    return all_files

def matches_glob_patterns(file_path: str, patterns: list[str]) -> bool:
    """Check if a file path matches any of the given glob patterns"""
    if not patterns:
        return False
    
    import fnmatch
    for pattern in patterns:
        if fnmatch.fnmatch(file_path, pattern):
            return True
    return False

def filter_files_by_patterns(
    files: list[str], allow_list: list[str] = None, avoid_list: list[str] = None
) -> list[str]:
    """Filter files based on allow and avoid glob patterns."""
    if not files:
        return files

    filtered_files = []

    for file_path in files:
        # Check avoid list first (takes priority)
        if avoid_list and matches_glob_patterns(file_path, avoid_list):
            continue

        # Check allow list
        if allow_list:
            if matches_glob_patterns(file_path, allow_list):
                filtered_files.append(file_path)
        else:
            # No allow list means all files are allowed (unless in avoid list)
            filtered_files.append(file_path)

    return filtered_files

def cloud_root_dir() -> str:
    from aird.constants import ROOT_DIR, CLOUD_SHARE_FOLDER
    return os.path.join(ROOT_DIR, CLOUD_SHARE_FOLDER)


def ensure_share_cloud_dir(share_id: str) -> str:
    share_dir = os.path.join(cloud_root_dir(), share_id)
    os.makedirs(share_dir, exist_ok=True)
    return share_dir


def sanitize_cloud_filename(name: str | None) -> str:
    candidate = (name or "cloud_file").strip()
    candidate = candidate.replace(os.sep, "_").replace("/", "_")
    candidate = re.sub(r"[^A-Za-z0-9._-]", "_", candidate)
    candidate = candidate.strip("._")
    if not candidate:
        candidate = "cloud_file"
    return candidate[:128]


def is_cloud_relative_path(share_id: str, relative_path: str) -> bool:
    from aird.constants import CLOUD_SHARE_FOLDER
    normalized = relative_path.replace("\\", "/")
    prefix = f"{CLOUD_SHARE_FOLDER}/{share_id}/"
    return normalized.startswith(prefix)


def remove_cloud_file_if_exists(share_id: str, relative_path: str) -> None:
    from aird.constants import ROOT_DIR
    if not is_cloud_relative_path(share_id, relative_path):
        return
    abs_path = os.path.abspath(os.path.join(ROOT_DIR, relative_path))
    if not is_within_root(abs_path, ROOT_DIR):
        return
    if os.path.isfile(abs_path):
        try:
            os.remove(abs_path)
        except OSError:
            pass
    cleanup_share_cloud_dir_if_empty(share_id)


def cleanup_share_cloud_dir_if_empty(share_id: str) -> None:
    import shutil
    share_dir = os.path.join(cloud_root_dir(), share_id)
    try:
        if os.path.isdir(share_dir) and not os.listdir(share_dir):
            shutil.rmtree(share_dir, ignore_errors=True)
    except Exception:
        pass


def remove_share_cloud_dir(share_id: str) -> None:
    import shutil
    if not share_id:
        return
    share_dir = os.path.join(cloud_root_dir(), share_id)
    shutil.rmtree(share_dir, ignore_errors=True)


def download_cloud_item(share_id: str, item: dict) -> str:
    from aird.cloud import CloudProviderError, CLOUD_MANAGER
    provider_name = item.get("provider")
    file_id = item.get("id")
    if not provider_name or not file_id:
        raise CloudProviderError("Invalid cloud file specification")
    if item.get("is_dir"):
        raise CloudProviderError("Cloud folder sharing is not supported")
    provider = CLOUD_MANAGER.get(provider_name)
    if not provider:
        raise CloudProviderError(f"Cloud provider '{provider_name}' is not configured")
    try:
        download = provider.download_file(file_id)
    except CloudProviderError:
        raise
    except Exception as exc:
        raise CloudProviderError(str(exc)) from exc

    filename = sanitize_cloud_filename(item.get("name") or getattr(download, "name", None) or f"{provider_name}-{file_id}")
    share_dir = ensure_share_cloud_dir(share_id)
    base, ext = os.path.splitext(filename)
    candidate = filename
    dest_path = os.path.join(share_dir, candidate)
    counter = 1
    while os.path.exists(dest_path):
        candidate = f"{base}_{counter}{ext}"
        dest_path = os.path.join(share_dir, candidate)
        counter += 1

    try:
        with open(dest_path, "wb") as fh:
            for chunk in download.iter_chunks():
                fh.write(chunk)
    except Exception as exc:
        try:
            os.remove(dest_path)
        except OSError:
            pass
        raise CloudProviderError(f"Failed to download cloud file '{filename}': {exc}") from exc
    finally:
        try:
            download.close()
        except Exception:
            pass
    from aird.constants import CLOUD_SHARE_FOLDER
    relative = join_path(CLOUD_SHARE_FOLDER, share_id, os.path.basename(dest_path))
    return relative


def download_cloud_items(share_id: str, items: list[dict]) -> list[str]:
    from aird.cloud import CloudProviderError
    if not items:
        return []
    downloaded: list[str] = []
    try:
        for item in items:
            downloaded.append(download_cloud_item(share_id, item))
        return downloaded
    except CloudProviderError:
        for rel_path in downloaded:
            remove_cloud_file_if_exists(share_id, rel_path)
        raise
    except Exception as exc:
        for rel_path in downloaded:
            remove_cloud_file_if_exists(share_id, rel_path)
        raise CloudProviderError(str(exc)) from exc

# Dummy implementations for parse_expression and evaluate_expression
# These should be replaced with a real implementation if complex filtering is needed.
def parse_expression(expression: str):
    """
    Parses a filter expression.
    For now, it returns a simple structure that `evaluate_expression` can use.
    """
    return {"raw": expression}

def evaluate_expression(line: str, parsed_expr: dict) -> bool:
    """
    Evaluates a parsed expression against a line of text.
    For now, it performs a simple case-insensitive substring search.
    """
    term = parsed_expr.get("raw", "")
    if not term:
        return True  # No filter means everything matches
    return term.lower() in line.lower()

def get_current_feature_flags() -> dict:
    """Return current feature flags with in-memory changes taking precedence over DB.
    This ensures real-time updates are immediately reflected.
    Falls back to in-memory defaults if DB is unavailable.
    """
    from aird.constants import FEATURE_FLAGS
    import aird.constants as constants_module
    from aird.db import _load_feature_flags

    # Start with in-memory flags (which may have just been updated)
    current = FEATURE_FLAGS.copy()
    
    db_conn = constants_module.DB_CONN
    if db_conn is not None:
        try:
            persisted = _load_feature_flags(db_conn)
            if persisted:
                # Start with DB values as base
                merged = persisted.copy()
                # Then overlay in-memory changes (in-memory takes precedence for real-time updates)
                for k, v in current.items():
                    merged[k] = bool(v)
                # Also include any DB-only flags
                for k, v in persisted.items():
                    if k not in merged:
                        merged[k] = bool(v)
                return merged
        except Exception:
            pass
    return current

def get_current_websocket_config() -> dict:
    """Return current WebSocket configuration with SQLite values taking precedence.
    Falls back to in-memory defaults if DB is unavailable.
    """
    from aird.constants import WEBSOCKET_CONFIG
    from aird.db import DB_CONN, _load_websocket_config

    current = WEBSOCKET_CONFIG.copy()
    if DB_CONN is not None:
        try:
            persisted = _load_websocket_config(DB_CONN)
            if persisted:
                # Persisted values override runtime defaults
                for k, v in persisted.items():
                    current[k] = int(v)
        except Exception:
            pass
    return current

def is_feature_enabled(key: str, default: bool = False) -> bool:
    flags = get_current_feature_flags()
    return bool(flags.get(key, default))


class MMapFileHandler:
    """Efficient file handling using memory mapping for large files"""
    
    @staticmethod
    def should_use_mmap(file_size: int) -> bool:
        """Determine if mmap should be used based on file size"""
        return file_size >= MMAP_MIN_SIZE
    
    @staticmethod
    async def serve_file_chunk(file_path: str, start: int = 0, end: int = None, chunk_size: int = CHUNK_SIZE):
        """Serve file chunks using mmap for efficient memory usage"""
        try:
            file_size = os.path.getsize(file_path)
            
            if not MMapFileHandler.should_use_mmap(file_size):
                # Use traditional method for small files
                with open(file_path, 'rb') as f:
                    f.seek(start)
                    remaining = (end - start + 1) if end is not None else file_size - start
                    while remaining > 0:
                        chunk = f.read(min(chunk_size, remaining))
                        if not chunk:
                            break
                        yield chunk
                        remaining -= len(chunk)
                return
            
            # Use mmap for large files
            with open(file_path, 'rb') as f:
                with mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ) as mm:
                    actual_end = min(end or file_size - 1, file_size - 1)
                    current = start
                    
                    while current <= actual_end:
                        chunk_end = min(current + chunk_size, actual_end + 1)
                        yield mm[current:chunk_end]
                        current = chunk_end
                        
        except (OSError, ValueError) as e:
            # Fallback to traditional method on mmap errors
            with open(file_path, 'rb') as f:
                f.seek(start)
                remaining = (end - start + 1) if end is not None else file_size - start
                while remaining > 0:
                    chunk = f.read(min(chunk_size, remaining))
                    if not chunk:
                        break
                    yield chunk
                    remaining -= len(chunk)
    
    @staticmethod
    def find_line_offsets(file_path: str, max_lines: int = None) -> list[int]:
        """Efficiently find line start offsets using mmap"""
        try:
            file_size = os.path.getsize(file_path)
            if not MMapFileHandler.should_use_mmap(file_size):
                # Use traditional method for small files
                offsets = [0]
                with open(file_path, 'rb') as f:
                    pos = 0
                    for line in f:
                        pos += len(line)
                        offsets.append(pos)
                        if max_lines and len(offsets) > max_lines:
                            break
                return offsets[:-1]  # Remove the last offset (EOF)
            
            # Use mmap for large files
            offsets = [0]
            with open(file_path, 'rb') as f:
                with mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ) as mm:
                    pos = 0
                    while pos < len(mm):
                        newline_pos = mm.find(b'\n', pos)
                        if newline_pos == -1:
                            break
                        pos = newline_pos + 1
                        offsets.append(pos)
                        if max_lines and len(offsets) > max_lines:
                            break
            return offsets[:-1]
            
        except (OSError, ValueError):
            # Fallback to traditional method
            offsets = [0]
            with open(file_path, 'rb') as f:
                pos = 0
                for line in f:
                    pos += len(line)
                    offsets.append(pos)
                    if max_lines and len(offsets) > max_lines:
                        break
            return offsets[:-1]
    
    @staticmethod
    def search_in_file(file_path: str, search_term: str, max_results: int = 100) -> list[dict]:
        """Efficiently search for text in file using mmap"""
        results = []
        try:
            file_size = os.path.getsize(file_path)
            search_bytes = search_term.encode('utf-8')
            
            if not MMapFileHandler.should_use_mmap(file_size):
                # Use traditional method for small files
                with open(file_path, 'r', encoding='utf-8', errors='replace') as f:
                    for line_num, line in enumerate(f, 1):
                        if search_term in line:
                            results.append({
                                "line_number": line_num,
                                "line_content": line.rstrip('\n'),
                                "match_positions": [i for i in range(len(line)) if line[i:].startswith(search_term)]
                            })
                            if len(results) >= max_results:
                                break
                return results
            
            # Use mmap for large files
            with open(file_path, 'rb') as f:
                with mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ) as mm:
                    current_pos = 0
                    line_number = 1
                    line_start = 0
                    
                    while current_pos < len(mm) and len(results) < max_results:
                        newline_pos = mm.find(b'\n', current_pos)
                        if newline_pos == -1:
                            # Last line
                            line_bytes = mm[current_pos:]
                            if search_bytes in line_bytes:
                                line_content = line_bytes.decode('utf-8', errors='replace')
                                match_positions = []
                                start_pos = 0
                                while True:
                                    pos = line_content.find(search_term, start_pos)
                                    if pos == -1:
                                        break
                                    match_positions.append(pos)
                                    start_pos = pos + 1
                                results.append({
                                    "line_number": line_number,
                                    "line_content": line_content,
                                    "match_positions": match_positions
                                })
                            break
                        
                        line_bytes = mm[current_pos:newline_pos]
                        if search_bytes in line_bytes:
                            line_content = line_bytes.decode('utf-8', errors='replace')
                            match_positions = []
                            start_pos = 0
                            while True:
                                pos = line_content.find(search_term, start_pos)
                                if pos == -1:
                                    break
                                match_positions.append(pos)
                                start_pos = pos + 1
                            results.append({
                                "line_number": line_number,
                                "line_content": line_content,
                                "match_positions": match_positions
                            })
                        
                        current_pos = newline_pos + 1
                        line_number += 1
                        
        except (OSError, UnicodeDecodeError):
            # Fallback to traditional search
            with open(file_path, 'r', encoding='utf-8', errors='replace') as f:
                for line_num, line in enumerate(f, 1):
                    if search_term in line:
                        results.append({
                            "line_number": line_num,
                            "line_content": line.rstrip('\n'),
                            "match_positions": [i for i in range(len(line)) if line[i:].startswith(search_term)]
                        })
                        if len(results) >= max_results:
                            break
        
        return results



"""Tests for aird/core/filter_expression.py"""

import pytest
from aird.core.filter_expression import FilterExpression


class TestFilterExpressionBasic:
    """Basic tests for FilterExpression class"""
    
    def test_empty_expression_matches_all(self):
        """Test that empty expression matches everything"""
        fe = FilterExpression("")
        assert fe.matches("any line") is True
        assert fe.matches("") is True
    
    def test_whitespace_expression_matches_all(self):
        """Test that whitespace-only expression matches everything"""
        fe = FilterExpression("   ")
        assert fe.matches("any line") is True
    
    def test_none_expression(self):
        """Test handling None-like expressions"""
        fe = FilterExpression("")
        assert fe.parsed_expression is None
        assert fe.matches("anything") is True
    
    def test_simple_term_matching(self):
        """Test simple term matching"""
        fe = FilterExpression("hello")
        assert fe.matches("hello world") is True
        assert fe.matches("HELLO WORLD") is True  # case insensitive
        assert fe.matches("goodbye") is False
    
    def test_case_insensitive_matching(self):
        """Test that matching is case insensitive"""
        fe = FilterExpression("ERROR")
        assert fe.matches("error occurred") is True
        assert fe.matches("Error: something") is True
        assert fe.matches("An ERROR happened") is True


class TestFilterExpressionQuoted:
    """Tests for quoted expressions"""
    
    def test_double_quoted_expression(self):
        """Test double-quoted literal expression"""
        fe = FilterExpression('"hello world"')
        assert fe.matches("hello world") is True
        assert fe.matches("helloworld") is False
    
    def test_single_quoted_expression(self):
        """Test single-quoted literal expression"""
        fe = FilterExpression("'hello world'")
        assert fe.matches("hello world") is True
        assert fe.matches("helloworld") is False
    
    def test_escaped_expression(self):
        """Test escaped expression (backslash prefix)"""
        fe = FilterExpression("\\hello AND world")
        # Should be treated as literal "hello AND world"
        assert fe.matches("hello AND world") is True


class TestFilterExpressionLogicalOperators:
    """Tests for AND/OR logical operators"""
    
    def test_and_operator(self):
        """Test AND operator"""
        fe = FilterExpression("hello AND world")
        assert fe.matches("hello world") is True
        assert fe.matches("hello there world") is True
        assert fe.matches("hello") is False
        assert fe.matches("world") is False
    
    def test_or_operator(self):
        """Test OR operator"""
        fe = FilterExpression("hello OR world")
        assert fe.matches("hello") is True
        assert fe.matches("world") is True
        assert fe.matches("hello world") is True
        assert fe.matches("goodbye") is False
    
    def test_and_case_insensitive(self):
        """Test AND operator is case insensitive"""
        fe = FilterExpression("hello and world")
        assert fe.matches("hello world") is True
        
        fe2 = FilterExpression("hello AND world")
        assert fe2.matches("hello world") is True
    
    def test_or_case_insensitive(self):
        """Test OR operator is case insensitive"""
        fe = FilterExpression("hello or world")
        assert fe.matches("hello") is True
        
        fe2 = FilterExpression("hello OR world")
        assert fe2.matches("hello") is True
    
    def test_multiple_and_operators(self):
        """Test multiple AND operators"""
        fe = FilterExpression("a AND b AND c")
        assert fe.matches("a b c") is True
        assert fe.matches("c b a") is True
        assert fe.matches("a b") is False
    
    def test_multiple_or_operators(self):
        """Test multiple OR operators"""
        fe = FilterExpression("a OR b OR c")
        assert fe.matches("a") is True
        assert fe.matches("b") is True
        assert fe.matches("c") is True
        assert fe.matches("d") is False
    
    def test_mixed_and_or_precedence(self):
        """Test that OR has lower precedence than AND"""
        # "a AND b OR c" should be "(a AND b) OR c"
        fe = FilterExpression("a AND b OR c")
        assert fe.matches("a b") is True  # a AND b
        assert fe.matches("c") is True     # OR c
        assert fe.matches("a") is False    # just a, not enough
        assert fe.matches("b") is False    # just b, not enough


class TestFilterExpressionParentheses:
    """Tests for parenthesized expressions"""
    
    def test_simple_parentheses(self):
        """Test simple parenthesized expression - parentheses only work with logical operators"""
        # Without logical operators, parentheses are treated as literal characters
        fe = FilterExpression("(hello)")
        assert fe.matches("(hello)") is True  # matches literal "(hello)"
        assert fe.matches("hello") is False   # doesn't match without parens
        
        # With a logical operator, parentheses work for grouping
        fe2 = FilterExpression("(hello) AND world")
        assert fe2.matches("hello world") is True
        assert fe2.matches("(hello) world") is True
    
    def test_nested_parentheses_and(self):
        """Test nested parentheses with AND"""
        fe = FilterExpression("(a AND b)")
        assert fe.matches("a b") is True
        assert fe.matches("a") is False
    
    def test_nested_parentheses_or(self):
        """Test nested parentheses with OR"""
        fe = FilterExpression("(a OR b)")
        assert fe.matches("a") is True
        assert fe.matches("b") is True
        assert fe.matches("c") is False
    
    def test_complex_parentheses(self):
        """Test complex parenthesized expression"""
        # "(a OR b) AND c" - either a or b, and must have c
        fe = FilterExpression("(a OR b) AND c")
        assert fe.matches("a c") is True
        assert fe.matches("b c") is True
        assert fe.matches("a b c") is True
        assert fe.matches("a") is False
        assert fe.matches("c") is False


class TestFilterExpressionEdgeCases:
    """Edge case tests for FilterExpression"""
    
    def test_word_with_and_in_it(self):
        """Test that words containing AND are not split"""
        fe = FilterExpression("android")
        assert fe.matches("android phone") is True
        # "android" contains "and" but should be treated as single term
    
    def test_word_with_or_in_it(self):
        """Test that words containing OR are not split"""
        fe = FilterExpression("order")
        assert fe.matches("order placed") is True
        # "order" contains "or" but should be treated as single term
    
    def test_str_representation(self):
        """Test __str__ representation"""
        fe = FilterExpression("test expression")
        assert "test expression" in str(fe)
    
    def test_balanced_parentheses(self):
        """Test balanced parentheses detection"""
        fe = FilterExpression("((a AND b))")
        assert fe.matches("a b") is True
    
    def test_quotes_in_split(self):
        """Test that operators inside quotes are not recognized"""
        fe = FilterExpression('"a AND b"')
        # Should be treated as literal "a AND b"
        assert fe.matches("a AND b") is True
        assert fe.matches("a b") is False  # should not match without AND


class TestFilterExpressionInternalMethods:
    """Tests for internal methods of FilterExpression"""
    
    def test_is_balanced_parentheses_simple(self):
        """Test _is_balanced_parentheses with simple cases"""
        fe = FilterExpression("")
        assert fe._is_balanced_parentheses("()") is True
        assert fe._is_balanced_parentheses("(())") is True
        assert fe._is_balanced_parentheses("(()") is False
        assert fe._is_balanced_parentheses("())") is False
    
    def test_is_balanced_parentheses_with_quotes(self):
        """Test _is_balanced_parentheses ignores parens in quotes"""
        fe = FilterExpression("")
        assert fe._is_balanced_parentheses('"("') is True
        assert fe._is_balanced_parentheses("')'") is True
    
    def test_is_standalone_operator_static(self):
        """Test _is_standalone_operator_static method"""
        # Test at start of string
        result = FilterExpression._is_standalone_operator_static("AND test", 0, 3)
        assert result is True
        
        # Test in middle with spaces
        result = FilterExpression._is_standalone_operator_static("a AND b", 2, 5)
        assert result is True
        
        # Test without proper spacing
        result = FilterExpression._is_standalone_operator_static("aANDb", 1, 4)
        assert result is False
    
    def test_parse_term_quoted(self):
        """Test _parse_term with quoted terms"""
        fe = FilterExpression("")
        result = fe._parse_term('"test"')
        assert result == {'type': 'term', 'value': 'test'}
        
        result = fe._parse_term("'test'")
        assert result == {'type': 'term', 'value': 'test'}
    
    def test_parse_term_unquoted(self):
        """Test _parse_term with unquoted terms"""
        fe = FilterExpression("")
        result = fe._parse_term('test')
        assert result == {'type': 'term', 'value': 'test'}


class TestFilterExpressionSplitting:
    """Tests for expression splitting logic"""
    
    def test_split_respecting_parentheses(self):
        """Test _split_respecting_parentheses method"""
        fe = FilterExpression("")
        
        # Simple OR split
        result = fe._split_respecting_parentheses("a OR b", "OR")
        assert len(result) == 2
        assert "a" in result[0]
        assert "b" in result[1]
    
    def test_split_respecting_quotes(self):
        """Test that splitting respects quoted strings"""
        fe = FilterExpression("")
        
        # OR inside quotes should not split
        result = fe._split_respecting_parentheses('"a OR b"', "OR")
        assert len(result) == 1
    
    def test_split_respecting_nested_parens(self):
        """Test that splitting respects nested parentheses"""
        fe = FilterExpression("")
        
        # OR inside parens should not split at top level
        result = fe._split_respecting_parentheses("(a OR b) AND c", "AND")
        assert len(result) == 2


class TestFilterExpressionRealWorldExamples:
    """Real-world usage examples for FilterExpression"""
    
    def test_log_filtering_error_and_warning(self):
        """Test filtering logs for errors or warnings"""
        fe = FilterExpression("error OR warning")
        assert fe.matches("[ERROR] Something went wrong") is True
        assert fe.matches("[WARNING] Be careful") is True
        assert fe.matches("[INFO] All good") is False
    
    def test_log_filtering_specific_error(self):
        """Test filtering for specific error type"""
        fe = FilterExpression("error AND database")
        assert fe.matches("Database connection error occurred") is True
        assert fe.matches("File not found error") is False
    
    def test_complex_log_filter(self):
        """Test complex log filtering"""
        fe = FilterExpression("(error OR exception) AND NOT timeout")
        # This would require NOT support, which doesn't exist
        # Just test without NOT
        fe = FilterExpression("(error OR exception) AND database")
        assert fe.matches("Database exception thrown") is True
        assert fe.matches("Database error logged") is True
        assert fe.matches("Database timeout") is False



"""Tests for aird/main.py"""

import pytest
import sqlite3
import tempfile
import os
import sys
from datetime import datetime, timedelta
from unittest.mock import patch, MagicMock

# Import functions from main.py
from aird.main import (
    _get_data_dir,
    _init_db,
    _load_feature_flags,
    _save_feature_flags,
    _is_share_expired,
    _cleanup_expired_shares,
    _hash_password,
    _verify_password,
    _create_user,
    _get_user_by_username,
    _get_all_users,
    _search_users,
    _update_user,
    _delete_user,
    _authenticate_user,
    _assign_admin_privileges,
    _extract_username_from_dn,
)


@pytest.fixture
def db_conn():
    """Create an in-memory SQLite database for testing"""
    conn = sqlite3.connect(":memory:")
    _init_db(conn)
    yield conn
    conn.close()


class TestGetDataDir:
    """Tests for _get_data_dir function"""
    
    def test_get_data_dir_windows(self):
        """Test data directory on Windows"""
        with patch('os.name', 'nt'), \
             patch('os.environ.get', side_effect=lambda k, d=None: 'C:\\Users\\Test\\AppData\\Local' if k == 'LOCALAPPDATA' else d), \
             patch('os.path.expanduser', return_value='C:\\Users\\Test\\AppData\\Local'), \
             patch('os.makedirs'), \
             patch('os.path.join', side_effect=lambda *args: '\\'.join(args)):
            result = _get_data_dir()
            assert 'aird' in result
    
    def test_get_data_dir_macos(self):
        """Test data directory on macOS"""
        with patch('os.name', 'posix'), \
             patch('sys.platform', 'darwin'), \
             patch('os.path.expanduser', return_value='/Users/test/Library/Application Support'), \
             patch('os.makedirs'), \
             patch('os.path.join', side_effect=lambda *args: '/'.join(args)):
            result = _get_data_dir()
            assert 'aird' in result
    
    def test_get_data_dir_linux(self):
        """Test data directory on Linux"""
        with patch('os.name', 'posix'), \
             patch('sys.platform', 'linux'), \
             patch('os.environ.get', side_effect=lambda k, d=None: '/home/test/.local/share' if k == 'XDG_DATA_HOME' else d), \
             patch('os.path.expanduser', return_value='/home/test/.local/share'), \
             patch('os.makedirs'), \
             patch('os.path.join', side_effect=lambda *args: '/'.join(args)):
            result = _get_data_dir()
            assert 'aird' in result
    
    def test_get_data_dir_fallback(self):
        """Test data directory fallback on exception"""
        with patch('os.name', 'nt'), \
             patch('os.environ.get', side_effect=Exception("Error")), \
             patch('os.getcwd', return_value='/current/dir'):
            result = _get_data_dir()
            assert result == '/current/dir'


class TestInitDb:
    """Tests for _init_db function"""
    
    def test_init_db_creates_tables(self, db_conn):
        """Test that _init_db creates all required tables"""
        cursor = db_conn.execute("SELECT name FROM sqlite_master WHERE type='table'")
        tables = [row[0] for row in cursor.fetchall()]
        
        assert 'feature_flags' in tables
        assert 'shares' in tables
        assert 'users' in tables
        assert 'ldap_configs' in tables
        assert 'ldap_sync_log' in tables
    
    def test_init_db_adds_missing_columns(self):
        """Test that _init_db adds missing columns to shares table"""
        conn = sqlite3.connect(":memory:")
        # Create shares table without new columns
        conn.execute("""
            CREATE TABLE shares (
                id TEXT PRIMARY KEY,
                created TEXT NOT NULL,
                paths TEXT NOT NULL
            )
        """)
        conn.commit()
        
        _init_db(conn)
        
        cursor = conn.execute("PRAGMA table_info(shares)")
        columns = [row[1] for row in cursor.fetchall()]
        
        assert 'allowed_users' in columns
        assert 'secret_token' in columns
        assert 'share_type' in columns
        assert 'allow_list' in columns
        assert 'avoid_list' in columns
        assert 'expiry_date' in columns
        
        conn.close()


class TestLoadFeatureFlags:
    """Tests for _load_feature_flags function"""
    
    def test_load_empty_flags(self, db_conn):
        """Test loading when no flags exist"""
        result = _load_feature_flags(db_conn)
        assert result == {}
    
    def test_load_existing_flags(self, db_conn):
        """Test loading existing feature flags"""
        db_conn.execute("INSERT INTO feature_flags (key, value) VALUES ('flag1', 1)")
        db_conn.execute("INSERT INTO feature_flags (key, value) VALUES ('flag2', 0)")
        db_conn.commit()
        
        result = _load_feature_flags(db_conn)
        assert result == {'flag1': True, 'flag2': False}
    
    def test_load_flags_exception_returns_empty(self):
        """Test that exceptions return empty dict"""
        mock_conn = MagicMock()
        mock_conn.execute.side_effect = Exception("Database error")
        
        result = _load_feature_flags(mock_conn)
        assert result == {}


class TestSaveFeatureFlags:
    """Tests for _save_feature_flags function"""
    
    def test_save_new_flags(self, db_conn):
        """Test saving new feature flags"""
        flags = {'flag1': True, 'flag2': False}
        _save_feature_flags(db_conn, flags)
        
        result = _load_feature_flags(db_conn)
        assert result == flags
    
    def test_save_updates_existing_flags(self, db_conn):
        """Test that saving updates existing flags"""
        db_conn.execute("INSERT INTO feature_flags (key, value) VALUES ('flag1', 0)")
        db_conn.commit()
        
        _save_feature_flags(db_conn, {'flag1': True})
        
        result = _load_feature_flags(db_conn)
        assert result['flag1'] is True


class TestIsShareExpired:
    """Tests for _is_share_expired function"""
    
    def test_no_expiry_date_not_expired(self):
        """Test that None expiry date means not expired"""
        assert _is_share_expired(None) is False
        assert _is_share_expired("") is False
    
    def test_future_expiry_not_expired(self):
        """Test that future expiry date is not expired"""
        future_date = (datetime.now() + timedelta(days=1)).isoformat()
        assert _is_share_expired(future_date) is False
    
    def test_past_expiry_is_expired(self):
        """Test that past expiry date is expired"""
        past_date = (datetime.now() - timedelta(days=1)).isoformat()
        assert _is_share_expired(past_date) is True
    
    def test_expiry_with_z_suffix(self):
        """Test expiry date with Z suffix"""
        past_date = (datetime.now() - timedelta(days=1)).isoformat() + "Z"
        assert _is_share_expired(past_date) is True
    
    def test_invalid_expiry_format_not_expired(self):
        """Test that invalid date format returns False"""
        assert _is_share_expired("invalid-date") is False


class TestCleanupExpiredShares:
    """Tests for _cleanup_expired_shares function"""
    
    def test_cleanup_removes_expired_shares(self, db_conn):
        """Test that cleanup removes expired shares"""
        past_date = (datetime.now() - timedelta(days=1)).isoformat()
        future_date = (datetime.now() + timedelta(days=1)).isoformat()
        
        db_conn.execute(
            "INSERT INTO shares (id, created, paths, expiry_date) VALUES (?, ?, ?, ?)",
            ("expired1", "2024-01-01", '["/path1"]', past_date)
        )
        db_conn.execute(
            "INSERT INTO shares (id, created, paths, expiry_date) VALUES (?, ?, ?, ?)",
            ("expired2", "2024-01-01", '["/path2"]', past_date)
        )
        db_conn.execute(
            "INSERT INTO shares (id, created, paths, expiry_date) VALUES (?, ?, ?, ?)",
            ("valid", "2024-01-01", '["/path3"]', future_date)
        )
        db_conn.commit()
        
        deleted_count = _cleanup_expired_shares(db_conn)
        
        assert deleted_count == 2
        cursor = db_conn.execute("SELECT id FROM shares WHERE id = ?", ("expired1",))
        assert cursor.fetchone() is None
        cursor = db_conn.execute("SELECT id FROM shares WHERE id = ?", ("valid",))
        assert cursor.fetchone() is not None


class TestHashPassword:
    """Tests for _hash_password function"""
    
    def test_hash_returns_string(self):
        """Test that hash returns a string"""
        result = _hash_password("testpassword")
        assert isinstance(result, str)
        assert len(result) > 0
    
    def test_hash_different_for_same_password(self):
        """Test that hashing same password twice gives different results"""
        hash1 = _hash_password("testpassword")
        hash2 = _hash_password("testpassword")
        # Should be different due to salt
        assert hash1 != hash2


class TestVerifyPassword:
    """Tests for _verify_password function"""
    
    def test_verify_correct_password(self):
        """Test verifying correct password"""
        password = "testpassword123"
        password_hash = _hash_password(password)
        assert _verify_password(password, password_hash) is True
    
    def test_verify_wrong_password(self):
        """Test verifying wrong password"""
        password_hash = _hash_password("correct_password")
        assert _verify_password("wrong_password", password_hash) is False


class TestCreateUser:
    """Tests for _create_user function"""
    
    def test_create_user_success(self, db_conn):
        """Test creating a new user successfully"""
        user = _create_user(db_conn, "testuser", "password123")
        
        assert user['username'] == "testuser"
        assert user['role'] == "user"
        assert user['id'] is not None
    
    def test_create_user_custom_role(self, db_conn):
        """Test creating a user with custom role"""
        user = _create_user(db_conn, "adminuser", "password123", role="admin")
        assert user['role'] == "admin"
    
    def test_create_user_duplicate_username(self, db_conn):
        """Test that duplicate username raises ValueError"""
        _create_user(db_conn, "testuser", "password123")
        
        with pytest.raises(ValueError):
            _create_user(db_conn, "testuser", "different_password")


class TestGetUserByUsername:
    """Tests for _get_user_by_username function"""
    
    def test_get_existing_user(self, db_conn):
        """Test getting an existing user"""
        _create_user(db_conn, "testuser", "password123")
        
        user = _get_user_by_username(db_conn, "testuser")
        
        assert user is not None
        assert user['username'] == "testuser"
    
    def test_get_nonexistent_user(self, db_conn):
        """Test getting a non-existent user returns None"""
        user = _get_user_by_username(db_conn, "nonexistent")
        assert user is None


class TestGetAllUsers:
    """Tests for _get_all_users function"""
    
    def test_get_all_users_empty(self, db_conn):
        """Test getting users when none exist"""
        result = _get_all_users(db_conn)
        assert result == []
    
    def test_get_all_users_multiple(self, db_conn):
        """Test getting multiple users"""
        _create_user(db_conn, "user1", "pass1")
        _create_user(db_conn, "user2", "pass2")
        
        result = _get_all_users(db_conn)
        
        assert len(result) == 2
        usernames = [u['username'] for u in result]
        assert "user1" in usernames
        assert "user2" in usernames


class TestSearchUsers:
    """Tests for _search_users function"""
    
    def test_search_users_by_partial_username(self, db_conn):
        """Test searching users by partial username"""
        _create_user(db_conn, "john_doe", "pass1")
        _create_user(db_conn, "jane_doe", "pass2")
        _create_user(db_conn, "bob_smith", "pass3")
        
        result = _search_users(db_conn, "doe")
        
        assert len(result) == 2
        usernames = [u['username'] for u in result]
        assert "john_doe" in usernames
        assert "jane_doe" in usernames


class TestUpdateUser:
    """Tests for _update_user function"""
    
    def test_update_user_role(self, db_conn):
        """Test updating user role"""
        user = _create_user(db_conn, "testuser", "pass1")
        
        result = _update_user(db_conn, user['id'], role="admin")
        
        assert result is True
        updated = _get_user_by_username(db_conn, "testuser")
        assert updated['role'] == "admin"
    
    def test_update_user_no_valid_fields(self, db_conn):
        """Test updating with no valid fields returns False"""
        user = _create_user(db_conn, "testuser", "pass1")
        
        result = _update_user(db_conn, user['id'], invalid_field="value")
        
        assert result is False
    
    def test_update_user_password(self, db_conn):
        """Test updating user password via 'password' field"""
        user = _create_user(db_conn, "testuser", "old_password")
        old_hash = _get_user_by_username(db_conn, "testuser")['password_hash']
        
        result = _update_user(db_conn, user['id'], password="new_password")
        
        assert result is True
        updated = _get_user_by_username(db_conn, "testuser")
        # Password hash should have changed
        assert updated['password_hash'] != old_hash
        # New password should verify correctly
        assert _verify_password("new_password", updated['password_hash']) is True


class TestDeleteUser:
    """Tests for _delete_user function"""
    
    def test_delete_existing_user(self, db_conn):
        """Test deleting an existing user"""
        user = _create_user(db_conn, "testuser", "pass1")
        
        result = _delete_user(db_conn, user['id'])
        
        assert result is True
        assert _get_user_by_username(db_conn, "testuser") is None
    
    def test_delete_nonexistent_user(self, db_conn):
        """Test deleting a non-existent user returns False"""
        result = _delete_user(db_conn, 99999)
        assert result is False


class TestAuthenticateUser:
    """Tests for _authenticate_user function"""
    
    def test_authenticate_valid_credentials(self, db_conn):
        """Test authenticating with valid credentials"""
        _create_user(db_conn, "testuser", "password123")
        
        result = _authenticate_user(db_conn, "testuser", "password123")
        
        assert result is not None
        assert result['username'] == "testuser"
    
    def test_authenticate_wrong_password(self, db_conn):
        """Test authenticating with wrong password"""
        _create_user(db_conn, "testuser", "password123")
        
        result = _authenticate_user(db_conn, "testuser", "wrongpassword")
        
        assert result is None
    
    def test_authenticate_nonexistent_user(self, db_conn):
        """Test authenticating non-existent user"""
        result = _authenticate_user(db_conn, "nonexistent", "password123")
        assert result is None


class TestAssignAdminPrivileges:
    """Tests for _assign_admin_privileges function"""
    
    def test_assign_admin_to_existing_user(self, db_conn):
        """Test assigning admin privileges to existing user"""
        _create_user(db_conn, "testuser", "password123")
        
        _assign_admin_privileges(db_conn, ["testuser"])
        
        user = _get_user_by_username(db_conn, "testuser")
        assert user['role'] == "admin"
    
    def test_assign_admin_empty_list(self, db_conn):
        """Test with empty list does nothing"""
        _create_user(db_conn, "testuser", "password123")
        
        _assign_admin_privileges(db_conn, [])
        
        user = _get_user_by_username(db_conn, "testuser")
        assert user['role'] == "user"


class TestExtractUsernameFromDn:
    """Tests for _extract_username_from_dn function"""
    
    def test_extract_username_simple_dn(self):
        """Test extracting username from simple DN"""
        dn = "uid=john.doe,ou=users,dc=example,dc=com"
        template = "uid={username},ou=users,dc=example,dc=com"
        
        result = _extract_username_from_dn(dn, template)
        assert result == "john.doe"
    
    def test_extract_username_complex_dn(self):
        """Test extracting username from complex DN with CN"""
        dn = "CN=John Doe,OU=Users,DC=example,DC=com"
        template = "CN={username},OU=Users,DC=example,DC=com"
        
        result = _extract_username_from_dn(dn, template)
        # Function looks for 'cn' (lowercase) in DN parts
        # It will find 'CN=John Doe' and extract 'John Doe'
        assert result == "John Doe"
    
    def test_extract_username_samaccountname(self):
        """Test extracting username with sAMAccountName"""
        dn = "sAMAccountName=john,CN=Users,DC=example,DC=com"
        template = "sAMAccountName={username},CN=Users,DC=example,DC=com"
        
        result = _extract_username_from_dn(dn, template)
        assert result == "john"
    
    def test_extract_username_no_match(self):
        """Test extracting username when DN doesn't have common attributes"""
        dn = "ou=users,dc=example,dc=com"
        template = "ou={username},dc=example,dc=com"
        
        result = _extract_username_from_dn(dn, template)
        # Should return None if no uid/cn/sAMAccountName found
        assert result is None



"""File operation utilities for scanning, filtering, and cloud file management."""

import os
import re
import shutil
import fnmatch
from aird.constants import ROOT_DIR, CLOUD_SHARE_FOLDER, CLOUD_MANAGER
from aird.core.security import is_within_root
from aird.cloud import CloudProviderError


def get_all_files_recursive(root_path: str, base_path: str = "") -> list:
    """Recursively get all files in a directory"""
    all_files = []
    try:
        for item in os.listdir(root_path):
            item_path = os.path.join(root_path, item)
            relative_path = os.path.join(base_path, item) if base_path else item
            
            if os.path.isfile(item_path):
                # It's a file, add it to the list
                all_files.append(relative_path)
            elif os.path.isdir(item_path):
                # It's a directory, recursively scan it
                sub_files = get_all_files_recursive(item_path, relative_path)
                all_files.extend(sub_files)
    except (OSError, PermissionError) as e:
        print(f"Error scanning directory {root_path}: {e}")
    
    return all_files


def matches_glob_patterns(file_path: str, patterns: list[str]) -> bool:
    """Check if a file path matches any of the given glob patterns"""
    if not patterns:
        return False
    
    for pattern in patterns:
        if fnmatch.fnmatch(file_path, pattern):
            return True
    return False


def filter_files_by_patterns(
    files: list[str], allow_list: list[str] = None, avoid_list: list[str] = None
) -> list[str]:
    """Filter files based on allow and avoid glob patterns."""
    if not files:
        return files

    filtered_files = []

    for file_path in files:
        # Check avoid list first (takes priority)
        if avoid_list and matches_glob_patterns(file_path, avoid_list):
            continue

        # Check allow list
        if allow_list:
            if matches_glob_patterns(file_path, allow_list):
                filtered_files.append(file_path)
        else:
            # No allow list means all files are allowed (unless in avoid list)
            filtered_files.append(file_path)

    return filtered_files


def cloud_root_dir() -> str:
    """Get the root directory for cloud file storage"""
    return os.path.join(ROOT_DIR, CLOUD_SHARE_FOLDER)


def ensure_share_cloud_dir(share_id: str) -> str:
    """Create and return the cloud directory for a specific share"""
    share_dir = os.path.join(cloud_root_dir(), share_id)
    os.makedirs(share_dir, exist_ok=True)
    return share_dir


def sanitize_cloud_filename(name: str | None) -> str:
    """Sanitize a filename for safe cloud storage"""
    candidate = (name or "cloud_file").strip()
    candidate = candidate.replace(os.sep, "_").replace("/", "_")
    candidate = re.sub(r"[^A-Za-z0-9._-]", "_", candidate)
    candidate = candidate.strip("._")
    if not candidate:
        candidate = "cloud_file"
    return candidate[:128]


def is_cloud_relative_path(share_id: str, relative_path: str) -> bool:
    """Check if a relative path is a cloud file path"""
    normalized = relative_path.replace("\\", "/")
    prefix = f"{CLOUD_SHARE_FOLDER}/{share_id}/"
    return normalized.startswith(prefix)


def remove_cloud_file_if_exists(share_id: str, relative_path: str) -> None:
    """Remove a cloud file if it exists"""
    if not is_cloud_relative_path(share_id, relative_path):
        return
    abs_path = os.path.abspath(os.path.join(ROOT_DIR, relative_path))
    if not is_within_root(abs_path, ROOT_DIR):
        return
    if os.path.isfile(abs_path):
        try:
            os.remove(abs_path)
        except OSError:
            pass
    cleanup_share_cloud_dir_if_empty(share_id)


def cleanup_share_cloud_dir_if_empty(share_id: str) -> None:
    """Remove share cloud directory if empty"""
    share_dir = os.path.join(cloud_root_dir(), share_id)
    try:
        if os.path.isdir(share_dir) and not os.listdir(share_dir):
            shutil.rmtree(share_dir, ignore_errors=True)
    except Exception:
        pass


def remove_share_cloud_dir(share_id: str) -> None:
    """Remove entire share cloud directory"""
    if not share_id:
        return
    share_dir = os.path.join(cloud_root_dir(), share_id)
    shutil.rmtree(share_dir, ignore_errors=True)


def download_cloud_item(share_id: str, item: dict) -> str:
    """Download a cloud item and return relative path"""
    provider_name = item.get("provider")
    file_id = item.get("id")
    if not provider_name or not file_id:
        raise CloudProviderError("Invalid cloud file specification")
    if item.get("is_dir"):
        raise CloudProviderError("Cloud folder sharing is not supported")
    provider = CLOUD_MANAGER.get(provider_name)
    if not provider:
        raise CloudProviderError(f"Cloud provider '{provider_name}' is not configured")
    try:
        download = provider.download_file(file_id)
    except CloudProviderError:
        raise
    except Exception as exc:
        raise CloudProviderError(str(exc)) from exc

    filename = sanitize_cloud_filename(item.get("name") or getattr(download, "name", None) or f"{provider_name}-{file_id}")
    share_dir = ensure_share_cloud_dir(share_id)
    base, ext = os.path.splitext(filename)
    candidate = filename
    dest_path = os.path.join(share_dir, candidate)
    counter = 1
    while os.path.exists(dest_path):
        candidate = f"{base}_{counter}{ext}"
        dest_path = os.path.join(share_dir, candidate)
        counter += 1

    with open(dest_path, "wb") as out:
        for chunk in download.iter_chunks():
            out.write(chunk)

    relative_path = os.path.relpath(dest_path, ROOT_DIR).replace("\\", "/")
    return relative_path


def download_cloud_items(share_id: str, items: list[dict]) -> list[str]:
    """Download multiple cloud items and return list of relative paths"""
    relative_paths = []
    for item in items:
        try:
            rel_path = download_cloud_item(share_id, item)
            relative_paths.append(rel_path)
        except CloudProviderError as e:
            print(f"Failed to download cloud item: {e}")
    return relative_paths


def configure_cloud_providers(config: dict | None) -> None:
    """Configure cloud providers from config"""
    if not config:
        return

    gd_config = config.get("google_drive")
    if gd_config and gd_config.get("enabled"):
        from aird.cloud import GoogleDriveProvider
        credentials_file = gd_config.get("credentials_file")
        token_file = gd_config.get("token_file")
        if credentials_file:
            try:
                gd = GoogleDriveProvider(credentials_file=credentials_file, token_file=token_file)
                CLOUD_MANAGER.register(gd)
                print("✅ Google Drive provider registered successfully")
            except Exception as e:
                print(f"⚠️  Failed to register Google Drive provider: {e}")
        else:
            print("⚠️  Google Drive enabled but missing 'credentials_file'")

    od_config = config.get("onedrive")
    if od_config and od_config.get("enabled"):
        from aird.cloud import OneDriveProvider
        client_id = od_config.get("client_id")
        client_secret = od_config.get("client_secret")
        redirect_uri = od_config.get("redirect_uri")
        token_file = od_config.get("token_file")
        if client_id and redirect_uri:
            try:
                od = OneDriveProvider(
                    client_id=client_id,
                    client_secret=client_secret,
                    redirect_uri=redirect_uri,
                    token_file=token_file
                )
                CLOUD_MANAGER.register(od)
                print("✅ OneDrive provider registered successfully")
            except Exception as e:
                print(f"⚠️  Failed to register OneDrive provider: {e}")
        else:
            print("⚠️  OneDrive enabled but missing 'client_id' or 'redirect_uri'")



import json
import os

import pytest
from unittest.mock import AsyncMock, MagicMock, patch

from aird.handlers.api_handlers import (
    FeatureFlagSocketHandler,
    FileListAPIHandler,
    FileStreamHandler,
    ShareDetailsAPIHandler,
    ShareDetailsByIdAPIHandler,
    ShareListAPIHandler,
    SuperSearchHandler,
    SuperSearchWebSocketHandler,
    UserSearchAPIHandler,
    WebSocketStatsHandler,
)
from tests.handler_helpers import authenticate, patch_db_conn, prepare_handler


def make_request_handler(handler_cls):
    """Helper for BaseHandler descendants."""
    app = MagicMock()
    app.settings = {'cookie_secret': 'test_secret'}
    request = MagicMock()
    request.protocol = "http"
    handler = prepare_handler(handler_cls(app, request))
    authenticate(handler, role='admin')
    return handler


def make_ws_handler(handler_cls):
    app = MagicMock()
    app.settings = {'cookie_secret': 'test_secret'}
    request = MagicMock()
    request.headers = {}
    request.path = "/ws"
    request.connection = MagicMock()
    handler = handler_cls(app, request)
    handler.write_message = MagicMock()
    handler.close = MagicMock()
    handler.request = request
    return handler


class TestFileListAPIHandler:
    def test_get_success(self):
        handler = make_request_handler(FileListAPIHandler)
        with patch('os.path.abspath', return_value='/root/path'), \
             patch('aird.handlers.api_handlers.is_within_root', return_value=True), \
             patch('os.path.isdir', return_value=True), \
             patch('aird.handlers.api_handlers.get_files_in_directory', return_value=[{'name': 'file.txt'}]), \
             patch('aird.handlers.api_handlers.is_video_file', return_value=False), \
             patch('aird.handlers.api_handlers.is_audio_file', return_value=False), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.api_handlers']), \
             patch('aird.handlers.api_handlers.get_all_shares', return_value={'share': {'paths': ['path/file.txt']}}):

            handler.get("path")
            handler.write.assert_called()
            payload = handler.write.call_args[0][0]
            assert payload['files'][0]['name'] == 'file.txt'

    def test_get_forbidden_path(self):
        handler = make_request_handler(FileListAPIHandler)
        handler.set_status = MagicMock()
        with patch('os.path.abspath', return_value='/bad'), \
             patch('aird.handlers.api_handlers.is_within_root', return_value=False):
            handler.get("bad")
            handler.set_status.assert_called_with(403)
            handler.write.assert_called_with("Access denied")

    def test_missing_directory(self):
        handler = make_request_handler(FileListAPIHandler)
        handler.set_status = MagicMock()
        with patch('os.path.abspath', return_value='/root/path'), \
             patch('aird.handlers.api_handlers.is_within_root', return_value=True), \
             patch('os.path.isdir', return_value=False):
            handler.get("missing")
            handler.set_status.assert_called_with(404)
            handler.write.assert_called_with("Directory not found")

    def test_get_handles_exception(self):
        handler = make_request_handler(FileListAPIHandler)
        handler.set_status = MagicMock()
        with patch('os.path.abspath', return_value='/root/path'), \
             patch('aird.handlers.api_handlers.is_within_root', return_value=True), \
             patch('os.path.isdir', return_value=True), \
             patch('aird.handlers.api_handlers.get_files_in_directory', side_effect=RuntimeError("boom")):
            handler.get("path")
            handler.set_status.assert_called_with(500)
            handler.write.assert_called_with("boom")


class TestSuperSearchHandler:
    def test_get_renders_when_enabled(self):
        handler = make_request_handler(SuperSearchHandler)
        handler.get_argument = MagicMock(return_value="//nested//")
        with patch('aird.main.is_feature_enabled', return_value=True):
            handler.get()
            handler.render.assert_called()
            assert handler.render.call_args[1]['current_path'] == "nested"

    def test_get_feature_disabled(self):
        handler = make_request_handler(SuperSearchHandler)
        handler.set_status = MagicMock()
        with patch('aird.main.is_feature_enabled', return_value=False):
            handler.get()
            handler.set_status.assert_called_with(403)
            handler.write.assert_called_with("Feature disabled: Super Search is currently disabled by administrator")


class TestSuperSearchWebSocketHandler:
    def test_open_requires_auth(self):
        handler = make_ws_handler(SuperSearchWebSocketHandler)
        handler.get_current_user = MagicMock(return_value=None)
        handler.open()
        handler.write_message.assert_called()
        handler.close.assert_called_with(code=1008, reason="Authentication required")

    def test_open_connection_limit(self):
        handler = make_ws_handler(SuperSearchWebSocketHandler)
        handler.get_current_user = MagicMock(return_value={'username': 'admin'})
        with patch.object(SuperSearchWebSocketHandler.connection_manager, 'add_connection', return_value=False):
            handler.open()
            handler.close.assert_called_with(code=1013, reason="Connection limit exceeded")

    @pytest.mark.asyncio
    async def test_on_message_auth_failure(self):
        handler = make_ws_handler(SuperSearchWebSocketHandler)
        handler.get_current_user = MagicMock(return_value=None)
        await handler.on_message(json.dumps({'pattern': '*.py', 'search_text': 'foo'}))
        handler.close.assert_called_with(code=1008, reason="Authentication expired")


class TestUserSearchAPIHandler:
    def test_requires_db_connection(self):
        handler = make_request_handler(UserSearchAPIHandler)
        handler.set_status = MagicMock()
        handler.get_argument = MagicMock(return_value="bob")
        with patch_db_conn(None, modules=['aird.handlers.api_handlers']):
            handler.get()
            handler.set_status.assert_called_with(500)
            handler.write.assert_called_with({"error": "Database not available"})

    def test_returns_empty_for_short_query(self):
        handler = make_request_handler(UserSearchAPIHandler)
        handler.get_argument = MagicMock(return_value=" ")
        with patch_db_conn(MagicMock(), modules=['aird.handlers.api_handlers']):
            handler.get()
            handler.write.assert_called_with({"users": []})

    def test_handles_search_error(self):
        handler = make_request_handler(UserSearchAPIHandler)
        handler.set_status = MagicMock()
        handler.get_argument = MagicMock(return_value="bob")
        with patch_db_conn(MagicMock(), modules=['aird.handlers.api_handlers']), \
             patch('aird.handlers.api_handlers.search_users', side_effect=RuntimeError("nope")):
            handler.get()
            handler.set_status.assert_called_with(500)
            handler.write.assert_called_with({"error": "nope"})

    def test_search_success(self):
        handler = make_request_handler(UserSearchAPIHandler)
        handler.get_argument = MagicMock(return_value="alice")
        with patch_db_conn(MagicMock(), modules=['aird.handlers.api_handlers']), \
             patch('aird.handlers.api_handlers.search_users', return_value=[{'username': 'alice'}]):
            handler.get()
            handler.write.assert_called_with({"users": [{'username': 'alice'}]})


class TestShareDetailsAPIHandler:
    def _make_handler(self):
        handler = make_request_handler(ShareDetailsAPIHandler)
        handler.get_argument = MagicMock(return_value="file.txt")
        return handler

    def test_feature_disabled(self):
        handler = self._make_handler()
        handler.set_status = MagicMock()
        with patch('aird.main.is_feature_enabled', return_value=False):
            handler.get()
            handler.set_status.assert_called_with(403)
            handler.write.assert_called_with({"error": "File sharing is disabled"})

    def test_missing_path(self):
        handler = self._make_handler()
        handler.set_status = MagicMock()
        handler.get_argument = MagicMock(return_value="")
        with patch('aird.main.is_feature_enabled', return_value=True):
            handler.get()
            handler.set_status.assert_called_with(400)
            handler.write.assert_called_with({"error": "File path is required"})

    def test_db_missing(self):
        handler = self._make_handler()
        handler.set_status = MagicMock()
        with patch('aird.main.is_feature_enabled', return_value=True), \
             patch_db_conn(None, modules=['aird.handlers.api_handlers']):
            handler.get()
            handler.set_status.assert_called_with(500)
            handler.write.assert_called_with({"error": "Database connection not available"})

    def test_success(self):
        handler = self._make_handler()
        with patch('aird.main.is_feature_enabled', return_value=True), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.api_handlers']), \
             patch('aird.handlers.api_handlers.get_shares_for_path', return_value=[{'id': 's1', 'paths': ['file.txt']}]):
            handler.get()
            handler.write.assert_called()
            payload = handler.write.call_args[0][0]
            assert payload['shares'][0]['id'] == 's1'


class TestShareDetailsByIdAPIHandler:
    def _make_handler(self):
        handler = make_request_handler(ShareDetailsByIdAPIHandler)
        handler.get_argument = MagicMock(return_value="share1")
        return handler

    def test_missing_id(self):
        handler = self._make_handler()
        handler.set_status = MagicMock()
        handler.get_argument = MagicMock(return_value="")
        with patch('aird.main.is_feature_enabled', return_value=True):
            handler.get()
            handler.set_status.assert_called_with(400)
            handler.write.assert_called_with({"error": "Share ID is required"})

    def test_share_not_found(self):
        handler = self._make_handler()
        handler.set_status = MagicMock()
        with patch('aird.main.is_feature_enabled', return_value=True), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.api_handlers']), \
             patch('aird.handlers.api_handlers.get_share_by_id', return_value=None):
            handler.get()
            handler.set_status.assert_called_with(404)
            handler.write.assert_called_with({"error": "Share not found"})

    def test_success(self):
        handler = self._make_handler()
        share_data = {'id': 'share1', 'paths': [], 'secret_token': 'token'}
        with patch('aird.main.is_feature_enabled', return_value=True), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.api_handlers']), \
             patch('aird.handlers.api_handlers.get_share_by_id', return_value=share_data):
            handler.get()
            payload = handler.write.call_args[0][0]
            assert payload['share']['id'] == 'share1'


class TestShareListAPIHandler:
    def test_feature_disabled(self):
        handler = make_request_handler(ShareListAPIHandler)
        handler.set_status = MagicMock()
        with patch('aird.main.is_feature_enabled', return_value=False):
            handler.get()
            handler.set_status.assert_called_with(403)
            handler.write.assert_called_with({"error": "File sharing is disabled"})

    def test_db_missing(self):
        handler = make_request_handler(ShareListAPIHandler)
        handler.set_status = MagicMock()
        with patch('aird.main.is_feature_enabled', return_value=True), \
             patch_db_conn(None, modules=['aird.handlers.api_handlers']):
            handler.get()
            handler.set_status.assert_called_with(500)
            handler.write.assert_called_with({"error": "Database connection not available"})

    def test_success(self):
        handler = make_request_handler(ShareListAPIHandler)
        with patch('aird.main.is_feature_enabled', return_value=True), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.api_handlers']), \
             patch('aird.handlers.api_handlers.get_all_shares', return_value={'s1': {'paths': []}}):
            handler.get()
            handler.write.assert_called_with({"shares": {'s1': {'paths': []}}})


class TestFeatureFlagSocketHandler:
    def test_open_sends_current_flags(self):
        handler = make_ws_handler(FeatureFlagSocketHandler)
        with patch.object(FeatureFlagSocketHandler.connection_manager, 'add_connection', return_value=True), \
             patch('aird.constants.FEATURE_FLAGS', {'in_memory': True}), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.api_handlers']), \
             patch('aird.handlers.api_handlers.load_feature_flags', return_value={'persisted': False}):
            
            # Mock authentication
            handler.get_current_user = MagicMock(return_value={'username': 'admin'})
            
            handler.open()
            handler.write_message.assert_called()
            message = json.loads(handler.write_message.call_args[0][0])
            assert message['in_memory'] is True

    def test_open_connection_limit(self):
        handler = make_ws_handler(FeatureFlagSocketHandler)
        # Mock authentication
        handler.get_current_user = MagicMock(return_value={'username': 'admin'})
        
        with patch.object(FeatureFlagSocketHandler.connection_manager, 'add_connection', return_value=False):
            handler.open()
            handler.close.assert_called_with(code=1013, reason="Connection limit exceeded")

    def test_send_updates_merges_flags(self):
        with patch('aird.constants.FEATURE_FLAGS', {'feature_a': True}), \
             patch_db_conn(MagicMock(), modules=['aird.handlers.api_handlers']), \
             patch('aird.handlers.api_handlers.load_feature_flags', return_value={'feature_b': False}), \
             patch.object(FeatureFlagSocketHandler.connection_manager, 'broadcast_message') as mock_broadcast:
            FeatureFlagSocketHandler.send_updates()
            payload = json.loads(mock_broadcast.call_args[0][0])
            assert payload['feature_a'] is True
            assert payload['feature_b'] is False


class TestFileStreamHandler:
    @pytest.mark.asyncio
    async def test_open_requires_auth(self):
        handler = make_ws_handler(FileStreamHandler)
        handler.get_current_user = MagicMock(return_value=None)
        await handler.open("log.txt")
        handler.close.assert_called_with(code=1008, reason="Authentication required")

    @pytest.mark.asyncio
    async def test_open_connection_limit(self):
        handler = make_ws_handler(FileStreamHandler)
        handler.get_current_user = MagicMock(return_value={'username': 'user'})
        with patch.object(FileStreamHandler.connection_manager, 'add_connection', return_value=False):
            await handler.open("log.txt")
            handler.close.assert_called_with(code=1013, reason="Connection limit exceeded")

    @pytest.mark.asyncio
    async def test_on_message_invalid_json(self):
        handler = make_ws_handler(FileStreamHandler)
        handler.get_current_user = MagicMock(return_value={'username': 'user'})
        await handler.on_message("not-json")
        handler.write_message.assert_called_with(json.dumps({'type': 'error', 'message': 'Invalid JSON payload'}))

    @pytest.mark.asyncio
    async def test_on_message_missing_action(self):
        handler = make_ws_handler(FileStreamHandler)
        handler.get_current_user = MagicMock(return_value={'username': 'user'})
        await handler.on_message(json.dumps({}))
        handler.write_message.assert_called_with(json.dumps({'type': 'error', 'message': 'Invalid request: action is required'}))


class TestWebSocketStatsHandler:
    def test_requires_admin(self):
        handler = make_request_handler(WebSocketStatsHandler)
        handler.set_status = MagicMock()
        handler.is_admin_user = MagicMock(return_value=False)
        handler.get()
        handler.set_status.assert_called_with(403)
        handler.write.assert_called_with("Access denied: You don't have permission to perform this action")

    def test_returns_stats(self):
        handler = make_request_handler(WebSocketStatsHandler)
        handler.is_admin_user = MagicMock(return_value=True)
        stats = {'connections': 1}
        with patch.object(FeatureFlagSocketHandler.connection_manager, 'get_stats', return_value=stats), \
             patch.object(FileStreamHandler.connection_manager, 'get_stats', return_value=stats), \
             patch.object(SuperSearchWebSocketHandler.connection_manager, 'get_stats', return_value=stats):
            handler.get()
            handler.write.assert_called()
            payload = json.loads(handler.write.call_args[0][0])
            assert 'feature_flags' in payload

